###########################################################################################
#
# Utilities
#
###########################################################################################

readTab <- function(fileName,colName=T){
  data <- read.csv(fileName,header=colName,sep="\t",as.is=T)
}


#between 0 and 1
setBetween0and1 <- function(x){
  return(max(0,min(1,x)))
}

#check file existence and accessibility
fileExistsAndAccessible <- function(filePath){
  if (  file.access(filePath,4) == 0){
    T
  }else{
    F
  }
}

computeSegmentsOverlap <- function(S1start,S1end,S2start,S2end){
  if (is.na(S1start) || is.na(S1end) || is.na(S2start) || is.na(S2end)){return (0)}
  if (S2end <= S1start || S2start >= S1end ){ return (0) }
  if (S2start <= S1start && S2end <= S1end ){ return (S2end - S1start + 1) }
  if (S2start <= S1start && S2end >= S1end ){ return (S1end - S1start + 1) }
  if (S2start >= S1start && S2end <= S1end ){ return (S2end - S2start + 1) }
  if (S2start >= S1start && S2end >= S1end ){ return (S1end - S2start + 1) }
}

getSegmentsPos <- function(chr, initialPos, finalPos, segments,flank = 0){
  #select segments that intersect initialPos and finalPs
  goodSegs <- which( ( as.numeric(gsub("chr","",segments[,1])) == as.numeric(gsub("chr","",chr))) & (
      (as.numeric(segments[,2]) >= (as.numeric(initialPos) - flank) & as.numeric(segments[,2]) <= (as.numeric(finalPos) + flank) ) |
      (as.numeric(segments[,3]) >= (as.numeric(initialPos) - flank) & as.numeric(segments[,3]) <= (as.numeric(finalPos) + flank) ) |
      (as.numeric(segments[,2]) <= (as.numeric(initialPos) - flank) & as.numeric(segments[,3]) >= (as.numeric(finalPos) + flank) )))
  
  return(goodSegs)  
}


#inputList <- BPprostate.clonality.list
#inputList <- betaTable.list
#inputList <- perGeneAnalysis.list
#inputList <- aa
#inputList <- geneDataCN.Good.list
fromListToDF <- function(inputList){
  if (is.null(inputList)){return(NULL)}
  
  #check if some is null and remove
  nullPositions <- which(sapply(inputList,is.null))
  if (length(nullPositions) > 0){
    inputList <- inputList[-nullPositions]    
  }
  
  
  #inputList <- globalAdm.list
  firstEl <- inputList[[1]]  

  
  #inputList <- lapply(inputList, function(x){ cat(x$gene,i,"\n"); i<-i+1;matrix(unlist(x), ncol=ncol(x))} )
#   for(i in c(1:length(inputList))){
#     #cat(inputList[[i]]$Gene.id[1],i,"\n")    
#     matrix(unlist(inputList[[i]]), ncol=ncol(inputList[[i]]))
#   }
#   
  inputList <- lapply(inputList, function(x){ matrix(unlist(x), ncol=ncol(x))} )

  #outDF <- as.data.frame(do.call(rbind, inputList),stringsAsFactors=F)    
  outDF <- as.data.frame(do.call(rbind, inputList),stringsAsFactors=F)
  colnames(outDF) <-names(firstEl)
  
   for(idx in c(1:ncol(outDF))){
     if (class(firstEl[[idx]]) == "logical"){       
       if (is.na(firstEl[[idx]])){
         class(outDF[[idx]]) <- "numeric"
       }else if (outDF[1,idx] == "TRUE" ||outDF[1,idx] == "FALSE"  ){
        outDF[,idx] <- as.logical(outDF[,idx]) * 1      
       }
       class(outDF[[idx]]) <- "numeric"
     }else{
       class(outDF[[idx]]) <- class(firstEl[[idx]])
     }
   }
#    tt<-lapply(c(1:ncol(outDF)),function(idx){
#      class(outDF[[idx]]) <- class(firstEl[[idx]])
#    })
   

  #sapply(firstEl, class)
  #sapply(outDF, class)
  #f <- which(sapply(firstEl, class)=="factor")
#   for(i in f) {
#        lev <- levels(firstEl[[i]])
#        outDF[[i]] <- factor(as.integer(outDF[[i]]), levels=seq_along(lev), labels=lev)
#    }
  
  return(outDF)
}


######################################################################
######################################################################
#
# Sample analysis
#
######################################################################
######################################################################

##pure tumor probability mass function
pureTumorDistribution <- function(x,sampleSize,nref){
  if (x == sampleSize){
    return (nref)
  }
  if (x == 0){
    return (1-nref)
  }
  return (0)
}

##pure blood probability mass function
pureBloodDistribution <- function(x,sampleSize,prob){
  return (dbinom(x,sampleSize,prob))
}

#convolution function of pure tumor and pure blood
convolutionBeta <- function(cover,beta,prob,nref){
  bloodCover <- round(cover * beta)
  tumorCover <- cover - bloodCover
  size <- cover
  vals <- c(0:size)  
  
  if (tumorCover == cover){
    for( i in c(0:size)){
      vals[i+1] <- pureTumorDistribution(i,tumorCover,nref)
    }
    return (vals)  
  }
  
  if (bloodCover == cover){
    for( i in c(0:size)){
      vals[i+1] <- pureBloodDistribution(i,size,prob)
    }
    return (vals)  
  }
  
  for( i in c(0:size)){
    vals[i+1] <- (1-nref)*dbinom(i,bloodCover,prob)+nref*dbinom((i-tumorCover),bloodCover,prob)
  }
  return (vals)
}


#comulative distribution function of beta and nref
convolutionBetaCumulative <- function(x,cover,beta,prob,nref){
  conv <- convolutionBeta(cover,beta,prob,nref)
  convCum <- c(1:length(conv))
  convCum[1] <- conv[1]
  for(i in c(2:length(conv))){
    convCum[i] <- convCum[i-1] + conv[i]
  }
  tmp <- c(1:length(x))
  #i<-1
  for(i in c(1:length(x))){
    index <- ceiling(x[i])
    tmp[i] <- convCum[index+1]     
  }
  return(tmp)
}

#Function to minimize as KS test
Dstat <- function(x,sampleRef,cover,prob){
  test<-max(abs(ecdf(sampleRef)(c(0:cover))-
    convolutionBetaCumulative(c(0:cover),cover,x[1],prob,x[2])))
  return(as.numeric(test))
}

##############################
#give reference and alternative vector absolute value
getRefAndAltPercAbs<-function(sample){
  sampleRef <- apply(sample,1,function(x){ return(  as.numeric(x[as.character(x["ref"]) ] )) })
  sampleAlt <- apply(sample,1,function(x){ return(  as.numeric(x[as.character(x["alt"]) ] )) })
  
  return (list(ref = sampleRef, alt = sampleAlt, max = NA))  
}


#compute the coverage of a list of SNPs as the sum of ref and alt reads
computeCoverage <- function(sample){
  return( apply(sample,1,function(x){return ( as.numeric(x[toString(x["ref"])])+as.numeric(x[toString(x["alt"])] ) )    }) )       
}

computeAF <- function(sample){
  return( apply(sample,1,function(x){return ( as.numeric(x[toString(x["ref"])]) / ( as.numeric(x[toString(x["ref"])])+as.numeric(x[toString(x["alt"])] ) ) )    }) )       
}


################################
#normalize the sample to the maxminal covera value times stretch
normalizeSampleMaxCov <- function(sample,stretch=2){
  coverage <- computeCoverage(sample)
  maxCov <- max(coverage) 
  stretchVal <- stretch * maxCov    
  
#   lapply(c(1:dim(sample)[1]),function(i){
#     if (coverage[i] > 0){
#       sample[i,toString(sample[i,"ref"])] <- round((stretchVal / coverage[i]) * sample[i,toString(sample[i,"ref"])])
#       sample[i,toString(sample[i,"alt"])] <- stretchVal - sample[i,toString(sample[i,"ref"])]
#     }else{
#       sample[i,toString(sample[i,"ref"])] <- 0
#       sample[i,toString(sample[i,"alt"])] <- 0
#     }
#   })
#   
  for(i in c(1:dim(sample)[1])){    
    if (coverage[i] > 0){
      sample[i,toString(sample[i,"ref"])] <- round((stretchVal / coverage[i]) * sample[i,toString(sample[i,"ref"])])
      sample[i,toString(sample[i,"alt"])] <- stretchVal - sample[i,toString(sample[i,"ref"])]
    }else{
      sample[i,toString(sample[i,"ref"])] <- 0
      sample[i,toString(sample[i,"alt"])] <- 0
    }
  }
  return( sample )    
}

###################################
#Main estimation
#sample <- pileupSample[representativeSNPs,],Prob=0.5,psoAlg=T)
estimateContamination <- function(sample,Prob=0.5,psoAlg=T,stretch=2){  
  aa <- computeCoverage(sample)
  coverR <- mean(aa)
  sdR <- sd(aa)
  #normalize tumor sample
  SampleNormalized <-normalizeSampleMaxCov(sample,stretch)
  #compute vector of ref reads
  SampleNormalizedRefAbs <- getRefAndAltPercAbs(SampleNormalized)  
  #compute cover
  cover <- SampleNormalizedRefAbs$ref[1]+SampleNormalizedRefAbs$alt[1]
  #opt <- optim(c(0.5,0.5), Dstat,sampleRef = SampleNormalizedRefAbs$ref, 
  #             cover = cover, prob = Prob, control = list(fnscale=1))  
  opt <- NULL
  psopt <- NULL
  if (psoAlg){
    psopt <- psoptim(rep(NA,2), Dstat, 
                     sampleRef = SampleNormalizedRefAbs$ref, 
                     cover = cover, prob = Prob,lower = c(0,0), upper = c(1,1),
                     #control=list(maxit=5))
                     control=list(maxit=500))
  }
  return (list(o=opt,pso=psopt,tn=SampleNormalizedRefAbs,cov=coverR,sd=sdR))
}



#################################
#Get the heterozygous position in a blood sample
getHetPositions_minPerc <- function(NormalSample, TumorSample, minPercAltBase, minCoverage){
  Het_SNPs <- c()
    
  NormalSampleRef <- apply(NormalSample,1,function(x){ return(  as.numeric(x[as.character(x["ref"]) ] )) })
  NormalSampleAlt <- apply(NormalSample,1,function(x){ return(  as.numeric(x[as.character(x["alt"]) ] )) })
  NormalSampleCov <- NormalSampleRef + NormalSampleAlt
  NormalSampleRefPerc <- round(NormalSampleRef/NormalSampleCov,2)
  NormalSampleAltPerc <- 1 - NormalSampleRefPerc
  
  TumorSampleRef <- apply(TumorSample,1,function(x){ return(  as.numeric(x[as.character(x["ref"]) ] )) })
  TumorSampleAlt <- apply(TumorSample,1,function(x){ return(  as.numeric(x[as.character(x["alt"]) ] )) })
  TumorSampleCov <- TumorSampleRef + TumorSampleAlt
  
  return(names(which(NormalSampleCov >= minCoverage & NormalSampleRefPerc >= minPercAltBase & NormalSampleAltPerc >= minPercAltBase & TumorSampleCov >= 5 )))  
}


###############################################
##Compute the represenative SNPs set, that is, the set of blood heterozygous SNPs
##that are in the deleted region
getRepresentativeSNPs <- function(NormalSample,DeletionInterval,TumorSample, minPercAltBase, minCoverage){
    
  #Deletion positions
  DelPositions <- which((NormalSample[,1] == paste("chr",toString(DeletionInterval[1]),sep="")  |
    NormalSample[,1] == toString(DeletionInterval[1]) ) &
    NormalSample[,"pos" ] >= as.numeric(DeletionInterval[2]) &
    NormalSample[,"pos" ] <= as.numeric(DeletionInterval[3]))

  RepSNPs <- getHetPositions_minPerc(NormalSample[DelPositions,],TumorSample[DelPositions,],minPercAltBase,minCoverage)  
  
  return(which(TumorSample[,3] %in% RepSNPs))
}


###################################
#Per sample analysis functions without rep SNPs
ClonET.analyseMutation<-function(mutation, 
                                 NormalSample, 
                                 TumorSample, 
                                 minPercAltBase, 
                                 minCoverage){      
  
  cat(paste("[",Sys.time() ,"] Processing mutation ",paste(mutation,collapse=".") ,"\n",sep=""))
  
  data<-NULL
  #data$peak<-unlist(peak)
  data$mutation<-mutation
  
  if (repSNPs){
    representativeSNPs <-   which((NormalSample[,1] == paste("chr",toString(data$mutation[1]),sep="")  |
      NormalSample[,1] == toString(DeletionInterval[1]) ) &
      NormalSample[,"pos" ] >= as.numeric(data$mutation[2]) &
      NormalSample[,"pos" ] <= as.numeric(data$mutation[3]))
  }else{
    representativeSNPS <- getRepresentativeSNPs(NormalSample,data$mutation[1:3],TumorSample,minPercAltBase,minCoverage)
  }
  
  
  
  if (length(representativeSNPS) > 1){
    data$repSNPs <- representativeSNPS
    data$computing <- estimateContamination(TumorSample[representativeSNPS,],Prob=0.5,psoAlg=T)
  }
  return(data)
}




ClonET.analyseSample<-function(sample, #Name of the tumor sample to analyse
                        samplesInfo, #File with tumor/blood sample name
                        mutationList, #Bed list of mutations to analyse (4 column for the name)
                        pileup_dir, #Where PaPI pileup is located
                        analysis_dir, #Where to save the analysis of the sample
                        minPercAltBase, #Min % of alt bases in blood
                        minCoverage, #Min cov in blood
                        PaPI_Suffix, #suffix of pileup files
                        perSampleCores
                        ){      
  
  cat(paste("[",Sys.time() ,"] Processing sample ",sample,"\n",sep=""))
  #data structure to save information
  dataOut<-NULL
  
  #identify the sample in the samplesInfo
  sampleLine <- which(samplesInfo$Tumor.Bam.Name == sample)
  if (length(sampleLine) == 0){
    cat(paste("[",Sys.time() ,"] Error: sample ",sample," is not in the samplesInfo file\n",sep=""))
    return(NULL)
  }
  
  tumorPileupFileName <- paste(pileup_dir,samplesInfo$Tumor.Bam.Name[sampleLine],PaPI_Suffix,sep="")
  normalPileupFileName <- paste(pileup_dir,samplesInfo$Normal.Bam.Name[sampleLine],PaPI_Suffix,sep="")
  if (file.info(tumorPileupFileName)$size == 0 ){ 
    cat(paste("[",Sys.time() ,"] Empty tumor pileup file: ",tumorPileupFileName,"\n",sep=""))
    next 
  }
  if (file.info(normalPileupFileName)$size == 0 ){ 
    cat(paste("[",Sys.time() ,"] Empty normal pileup file: ",normalPileupFileName,"\n",sep=""))
    next     
  }
  #Load pileup files
  TumorSample <- read.csv(tumorPileupFileName,
                          sep="\t",check.names=F,row.names = NULL, as.is =T )
  NormalSample <- read.csv(normalPileupFileName,
                          sep="\t",check.names=F,row.names = NULL,  as.is =T )
    
  #remove lines where ref or alt has more than 1 value
  TumorSample <- TumorSample[apply(TumorSample,1, function(x){return( nchar(x[5]) == 1 && nchar(x[6]) == 1 )}),]
  NormalSample <- NormalSample[apply(NormalSample,1, function(x){return( nchar(x[5]) == 1 && nchar(x[6]) == 1 )}),]
  
  #Sort BloodSample as TumorSample using dbSNP ID
  rownames(NormalSample) <- NormalSample[,"dbsnp"]
  rownames(TumorSample) <- TumorSample[,"dbsnp"]
  NormalSample <- NormalSample[rownames(TumorSample),]
  rownames(NormalSample) <- NormalSample[,"dbsnp"]
        
  #save sample name
  dataOut$sampleName<-sample
  
  #there is data
  if ( nrow(NormalSample) > 0 & nrow(TumorSample) > 0 ){
    contaminationResult <- NULL
    contaminationResult <- mclapply(split(mutationList,1:dim(mutationList)[1]),
                                      ClonET.analyseMutation, 
                                      NormalSample = NormalSample, 
                                      TumorSample = TumorSample,                                       
                                      minPercAltBase = minPercAltBase,
                                      minCoverage = minCoverage,
                                      mc.cores= perSampleCores,
                                      mc.preschedule = T)    
    dataOut$results<- NULL
    dataOut$results<-contaminationResult
    dataOut$deletions<-mutationList
  }
  
  outName<-paste(analysis_dir,sample,".contaminationAnalysis.RData",sep="")
  save(dataOut,file=outName)
  
}

###################################
#Per sample analysis functions: preprocess rep SNPs


ClonET.getInformativeSNPs<-function(sample, #Name of the tumor sample to analyse
                                    samplesInfo, #File with tumor/blood sample name                           
                                    pileup_dir, #Where PaPI pileup is located
                                    infoSNPs_dir, #Where to save informative SNPs pileup
                                    minPercAltBase, #Min % of alt bases in blood
                                    minCoverage, #Min cov in blood
                                    PaPI_Suffix){      

  cat(paste("[",Sys.time() ,"] Processing sample ",sample,"\n",sep=""))
  #data structure to save information
    
  #identify the sample in the samplesInfo
  sampleLine <- which(samplesInfo$Tumor.Bam.Name == sample)
  if (length(sampleLine) == 0){
    cat(paste("[",Sys.time() ,"] Error: sample ",sample," is not in the samplesInfo file\n",sep=""))
    return()
  }
  
  tumorPileupFileName <- paste(pileup_dir,samplesInfo$Tumor.Bam.Name[sampleLine],PaPI_Suffix,sep="")
  normalPileupFileName <- paste(pileup_dir,samplesInfo$Normal.Bam.Name[sampleLine],PaPI_Suffix,sep="")
  if (file.info(tumorPileupFileName)$size == 0 ){ 
    cat(paste("[",Sys.time() ,"] Empty tumor pileup file: ",tumorPileupFileName,"\n",sep=""))
    return()
  }
  if (file.info(normalPileupFileName)$size == 0 ){ 
    cat(paste("[",Sys.time() ,"] Empty normal pileup file: ",normalPileupFileName,"\n",sep=""))
    return()
  }
  
  #return()
  #cat("Here")
  #Load pileup files
  TumorSample <- read.csv(tumorPileupFileName,
                          sep="\t",check.names=F,row.names = NULL, as.is =T )
  NormalSample <- read.csv(normalPileupFileName,
                           sep="\t",check.names=F,row.names = NULL,  as.is =T )
  
  #remove lines where ref or alt has more than 1 value
  TumorSample <- TumorSample[apply(TumorSample,1, function(x){return( nchar(x[5]) == 1 && nchar(x[6]) == 1 )}),]
  NormalSample <- NormalSample[apply(NormalSample,1, function(x){return( nchar(x[5]) == 1 && nchar(x[6]) == 1 )}),]
  
  #Sort BloodSample as TumorSample using dbSNP ID
  rownames(NormalSample) <- NormalSample[,"dbsnp"]
  rownames(TumorSample) <- TumorSample[,"dbsnp"]
  NormalSample <- NormalSample[rownames(TumorSample),]
  rownames(NormalSample) <- NormalSample[,"dbsnp"]

  
  informativeSNPs <- getHetPositions_minPerc(NormalSample=NormalSample, TumorSample=TumorSample, minPercAltBase=minPercAltBase, minCoverage=minCoverage )
  
  tumorSampleInformativeSNPs <- TumorSample[informativeSNPs,]
    
  
  outName<-paste(infoSNPs_dir,sample,".infoSNPs.pileup.vcf",sep="")
  write.table(tumorSampleInformativeSNPs, file=outName, sep="\t",row.names=F,col.names=T,quote=F )
  #save(dataOut,file=outName)
  
}


ClonET.analyseMutation.infoSNPs<-function(mutation, 
                                 pileupSample,                                  
                                 minPercAltBase, 
                                 minCoverage){      
  
  #cat(paste("[",Sys.time() ,"] Processing mutation ",paste(mutation,collapse=".") ,"\n",sep=""))
    
  data<-NULL
  #data$peak<-unlist(peak)
  data$mutation<-mutation
  
  
  representativeSNPs <-   which((
    pileupSample[,1] == paste("chr",toString(data$mutation[1]),sep="")  |
    pileupSample[,1] == toString(data$mutation[1]) ) &
    pileupSample[,"pos" ] >= as.numeric(data$mutation[2]) &
    pileupSample[,"pos" ] <= as.numeric(data$mutation[3]) )
     
  if (length(representativeSNPs) > 1){
    data$repSNPs <- pileupSample$dbsnp[representativeSNPs]
    data$computing <- estimateContamination(pileupSample[representativeSNPs,],Prob=0.5,psoAlg=T)
  }
  return(data)
}




ClonET.analyseMutation.infoSNPs.tryCatch<-function(mutation, 
                                                   pileupSample,                                  
                                                   minPercAltBase, 
                                                   minCoverage){     
  tryCatch(ClonET.analyseMutation.infoSNPs(mutation, 
                                           pileupSample,                                  
                                           minPercAltBase, 
                                           minCoverage), 
             error = function(e){ 
             #read current read
             mut<-paste0(mutation[1],":",mutation[2],"-",mutation[3])
             cat("Error on mutation ",mut,"\n"  )
           }
  )
}




ClonET.analyseSample.infoSNPs<-function(sample, #Name of the tumor sample to analyse
                                       samplesInfo, #File with tumor/blood sample name
                                       mutationList, #Bed list of mutations to analyse (4 column for the name)
                                       pileup_dir, #Where PaPI pileup is located
                                       analysis_dir, #Where to save the analysis of the sample
                                       minPercAltBase, #Min % of alt bases in blood
                                       minCoverage, #Min cov 
                                       PaPI_Suffix, #suffix of pileup files
                                       perSampleCores
){      
  
  cat(paste("[",Sys.time() ,"] Processing sample ",sample,"\n",sep=""))
  #data structure to save information
  dataOut<-NULL
  
  #identify the sample in the samplesInfo
  sampleLine <- which(samplesInfo$Tumor.Bam.Name == sample)  
  if (length(sampleLine) == 0){
    cat(paste("[",Sys.time() ,"] Error: sample ",sample," is not in the samplesInfo file\n",sep=""))
    return()
  }
  
  samplesInfo$Tumor.Bam.Name[sampleLine] <- gsub(".bam","",basename(samplesInfo$Tumor.Bam.Name[sampleLine]))
  sample <- gsub(".bam","",basename(samplesInfo$Tumor.Bam.Name[sampleLine]))
  
  pileupFileName <- paste(pileup_dir,samplesInfo$Tumor.Bam.Name[sampleLine],PaPI_Suffix,sep="")  
  if (!file.exists(pileupFileName)){
    cat(paste("[",Sys.time() ,"] Error: cannot find file  ",pileupFileName,"\n",sep=""))
    return()
  }
  
  #normalPileupFileName <- paste(pileup_dir,samplesInfo$Normal.Bam.Name[sampleLine],PaPI_Suffix,sep="")
  if (file.info(pileupFileName)$size == 0 ){ 
    cat(paste("[",Sys.time() ,"] Warning: empty pileup file ",pileupFileName,"\n",sep=""))
    return()
  }

  #Load pileup files
  pileupSample <- read.csv(pileupFileName, sep="\t",check.names=F,row.names = NULL, as.is =T )
  
  #remove lines where ref or alt has more than 1 value: DONE
  pileupSample <- pileupSample[apply(pileupSample,1, function(x){return( nchar(x[5]) == 1 && nchar(x[6]) == 1 )}),]

  #remove lines where cov is less than min cov  
  #pileupSample <- pileupSample[which(apply(pileupSample[,7:10],1,sum)!=0),]
  pileupSample <- pileupSample[which(apply(pileupSample[,7:10],1,sum)>=minCoverage),]
  
  
  #Sort BloodSample as TumorSample using dbSNP ID: DONE
  #rownames(NormalSample) <- NormalSample[,"dbsnp"]
  #rownames(TumorSample) <- TumorSample[,"dbsnp"]
  #NormalSample <- NormalSample[rownames(TumorSample),]
  #rownames(NormalSample) <- NormalSample[,"dbsnp"]
  
  #save sample name
  dataOut$sampleName <- sample
  
  #there is data
  contaminationResult <- NULL
  
  #if mutalionList has more than 4 columns, the 5^ is interpreted as sample name
  if (ncol(mutationList) > 4){
    #mutationSampleName <- gsub("-Tumor","",samplesInfo[sampleLine,"Tumor.Array.Name"])
    #mutationList <- mutationList[which(mutationList$V5 == mutationSampleName),]
    mutationList <- mutationList[which(mutationList$V5 == sample),]
    if ( nrow(mutationList) == 0 ){
      cat("Warning: no areas defined for sample ",sample,"\n")
      return()
    }
  }
  
  contaminationResult <- mclapply(split(mutationList,1:dim(mutationList)[1]),
                                    ClonET.analyseMutation.infoSNPs.tryCatch, 
                                    pileupSample = pileupSample,                                       
                                    minPercAltBase = minPercAltBase,
                                    minCoverage = minCoverage,
                                    mc.cores= perSampleCores,
                                    mc.preschedule = F)    
  dataOut$results<- NULL
  dataOut$results<-contaminationResult
  dataOut$deletions<-mutationList
  
  
  outName<-paste(analysis_dir,sample,".contaminationAnalysis.RData",sep="")
  #outName<- "/home/prandi/test.RData"
  save(dataOut,file=outName)
  
}


ClonET.analyseSample.infoSNPs.tryCatch<-function(sample, #Name of the tumor sample to analyse
                                        samplesInfo, #File with tumor/blood sample name
                                        mutationList, #Bed list of mutations to analyse (4 column for the name)
                                        pileup_dir, #Where PaPI pileup is located
                                        analysis_dir, #Where to save the analysis of the sample
                                        minPercAltBase, #Min % of alt bases in blood
                                        minCoverage, #Min cov in blood
                                        PaPI_Suffix, #suffix of pileup files
                                        perSampleCores
){
  tryCatch(ClonET.analyseSample.infoSNPs(sample, #Name of the tumor sample to analyse
                                samplesInfo, #File with tumor/blood sample name
                                mutationList, #Bed list of mutations to analyse (4 column for the name)
                                pileup_dir, #Where PaPI pileup is located
                                analysis_dir, #Where to save the analysis of the sample
                                minPercAltBase, #Min % of alt bases in blood
                                minCoverage, #Min cov in blood
                                PaPI_Suffix, #suffix of pileup files
                                perSampleCores
  ), 
        error = function(e){ 
             #read current read
             cat("Error on sample ",sample,"\n"  )
           }
  )
}


######################################################################
######################################################################
#
# Aggregate per sample analysis
#
######################################################################
######################################################################
aggregateSamplesAnalysis<-function(analysisFiles){

  outTable <- c()
  #analysis <- analysisFiles[2]
  for( analysis in analysisFiles){
     cat("Loading analysis",analysis,"\n")
     load(analysis) 
     cat("Done\n")
     sampleName <- dataOut$sampleName
     if ( length(dataOut$results) == 0) { next }
     #i<-1
     for(i in c(1:length(dataOut$results))){
        thisRes <- dataOut$results[i]
        if (is.null(thisRes[[1]]$computing)){
          #region <- thisRes[[1]]$mutation
          #pso <- c(NA,NA)
          #nsnps <- NA
          #cov <- NA
          next
        }else{
          region <- thisRes[[1]]$mutation
          pso <- round(thisRes[[1]]$computing$pso$par,2)
          nsnps <- length(thisRes[[1]]$repSNPs)
          cov <- round(thisRes[[1]]$computing$cov)
        }#cat(cov)
        outTable <- rbind(outTable, unlist(c(sampleName,region[,1:4],pso,nsnps,cov)))
        colnames(outTable) <- c("sample","chr","start","end","zone_id","beta","nref","n_snps","cov")
     }
     cat("Analysis sample ",analysis,"completed \n")
  }
  
  #colnames(outTable) <- c("sample","chr","start","end","zone_id","beta","nref","n_snps","cov_max")
  
  return(outTable)
}



######################################################################
######################################################################
#
# Functions for coverage computing
#
######################################################################
######################################################################

CLONET.computeLog2Ratio.Zone <- function(zoneData,
                                 tumorPileup,
                                 normalPileup,
                                 normalGlobalCoverage,
                                 tumorGlobalCoverage)
{
  zoneTumorSample <-   tumorPileup[which((tumorPileup[,1] == paste("chr",toString(zoneData$chr),sep="")  |
                                tumorPileup[,1] == toString(zoneData$chr) ) &
                               tumorPileup[,"pos" ] >= as.numeric(zoneData$start) &
                               tumorPileup[,"pos" ] <= as.numeric(zoneData$end))  ,]
  zoneNormalSample <- normalPileup[which((normalPileup[,1] == paste("chr",toString(zoneData$chr),sep="")  |
                                normalPileup[,1] == toString(zoneData$chr) ) &
                               normalPileup[,"pos" ] >= as.numeric(zoneData$start) &
                               normalPileup[,"pos" ] <= as.numeric(zoneData$end)),]
  
  zoneTumorMeanCov <- round(mean(apply(zoneTumorSample[7:10],1,sum)),3)
  zoneTumorSdCov <- round(sd(apply(zoneTumorSample[7:10],1,sum)),3)
  
  zoneNormalMeanCov <- round(mean(apply(zoneNormalSample[7:10],1,sum)),3)
  zoneNormalSdCov <- round(sd(apply(zoneNormalSample[7:10],1,sum)),3)
  
  log2Ratio <- round(log2( (zoneTumorMeanCov / zoneNormalMeanCov ) / ( tumorGlobalCoverage / normalGlobalCoverage )),3)

  zoneData$tumor.Mean.Cov <- zoneTumorMeanCov
  zoneData$tumor.SD.Cov <- zoneTumorSdCov

  zoneData$normal.Mean.Cov <- zoneNormalMeanCov
  zoneData$normal.SD.Cov <- zoneNormalSdCov  
  
  zoneData$log2ratio <- log2Ratio
  
  
  return(zoneData)
}



CLONET.computeLog2Ratio.Sample <- function(sampleData, #CLONET analysis for one sample
                                     pileup_dir, #dir for PaPI data
                                     PaPI_Suffix, #suffic of PaPI files                                    
                                     samplesInfo, #sample names conversion
                                     globalCoverage, #coverage of the sample
                                     ncoresXsample)#numer of cores for this sample
{
  
  #retrive global coverage
  tumorSample <- sampleData$sample[1]
  individualName <- gsub("-Tumor","",samplesInfo[tumorSample,"Tumor.Array.Name"])
  rownames(globalCoverage) <- globalCoverage$Sample
  tumorGlobalCoverage <- globalCoverage[individualName,"tdepth"]
  normalGlobalCoverage <- globalCoverage[individualName,"ndepth"]
  
  #load pileup data
  tumorPileupName <- paste(pileup_dir,tumorSample,PaPI_Suffix,sep="")
  tumorPileup <- read.csv(tumorPileupName,header=T,sep="\t",as.is=T,check.names=F)
  
  normalSample <- samplesInfo[tumorSample,"Normal.Bam.Name"]
  normalPileupName <- paste(pileup_dir,normalSample,PaPI_Suffix,sep="")
  normalPileup <- read.csv(normalPileupName,header=T,sep="\t",as.is=T,check.names=F)
  
  
  #there is data
  if ( nrow(normalPileup) > 0 & nrow(tumorPileup) > 0 ){  
    #zoneData <- split(sampleData,1:dim(sampleData)[1])[[7]]
    sampleDataLog2 <- mclapply(split(sampleData,1:dim(sampleData)[1]), 
                                    CLONET.computeLog2Ratio.Zone, 
                                    normalPileup = normalPileup, 
                                    tumorPileup = tumorPileup,
                                    normalGlobalCoverage = normalGlobalCoverage,
                                    tumorGlobalCoverage = tumorGlobalCoverage,
                                    mc.cores = ncoresXsample,
                                    mc.preschedule = T
                                    )
    
    sampleDataLog2 <- do.call(rbind,sampleDataLog2)

    return(sampleDataLog2)
  }else{
    return(NULL)
  }
}



######################################################################
######################################################################
#
# Functions for log2 data computing
#
######################################################################
######################################################################

getAdm <- function(beta,cnT,cnN){
  return( (cnT * beta ) / ((cnT - cnN) * beta + cnN ) )
}

getBeta <- function(adm,cnT,cnN){
    return( ( adm * cnN  ) / ( cnT + adm * cnN - adm * cnT  ) )
}



######################################################################
######################################################################
#
# Functions for breakpoints analysis
#
######################################################################
######################################################################



############
#functions will be moved to CLONET functions

#convert BP string in dRanger compatible vector
fromBPtodRanger <- function(s,seqlength=NULL){
  strSp <- strsplit(x=as.character(s),split="_")[[1]]
  
  leftBreakPointChr <- as.numeric(strSp[3])
  leftBreakPointPos <- as.numeric(strSp[4])
  rightBreakPointChr <- as.numeric(strSp[5])
  rightBreakPointPos <- as.numeric(strSp[6])
  
  return(c(leftBreakPointChr,leftBreakPointPos, rightBreakPointChr,rightBreakPointPos))
}

#s <- as.character(seqnames(readsRanges)[1])
#seqlength <- 101
fromBPtoBed <- function(s,seqlength){
  strSp <- strsplit(x=as.character(s),split="_")[[1]]
  
  leftBreakPointChr <- as.numeric(strSp[3])
  leftBreakPointPos <- as.numeric(strSp[4])
  rightBreakPointChr <- as.numeric(strSp[5])
  rightBreakPointPos <- as.numeric(strSp[6])
  
  return(rbind(c(leftBreakPointChr,leftBreakPointPos-seqlength+1,leftBreakPointPos ),
               c(rightBreakPointChr,rightBreakPointPos,rightBreakPointPos+seqlength-1 )))
}

#read can be on both side of a breakpoint
#s<-seqnames(testR)
#seqlength<-width(ranges(testR))
fromBPtoBedSimmetric <- function(s,seqlength){
  strSp <- strsplit(x=as.character(s),split="_")[[1]]
  
  leftBreakPointChr <- as.numeric(strSp[3])
  leftBreakPointPos <- as.numeric(strSp[4])
  rightBreakPointChr <- as.numeric(strSp[5])
  rightBreakPointPos <- as.numeric(strSp[6])
  
  return(rbind(c(leftBreakPointChr,leftBreakPointPos-seqlength+1,leftBreakPointPos ),
               c(rightBreakPointChr,rightBreakPointPos-seqlength+1,rightBreakPointPos ),
               c(rightBreakPointChr,rightBreakPointPos,rightBreakPointPos+seqlength-1 ),
               c(leftBreakPointChr,leftBreakPointPos,leftBreakPointPos+seqlength-1 )
  ))
}


fromBPtoBedLarge <- function(s,seqlength,flank){
  strSp <- strsplit(x=as.character(s),split="_")[[1]]
  
  leftBreakPointChr <- as.numeric(strSp[3])
  leftBreakPointPos <- as.numeric(strSp[4])
  rightBreakPointChr <- as.numeric(strSp[5])
  rightBreakPointPos <- as.numeric(strSp[6])
  
  return(rbind(c(leftBreakPointChr,leftBreakPointPos-seqlength+1-flank,leftBreakPointPos+seqlength-1+flank),
               c(rightBreakPointChr,rightBreakPointPos-seqlength+1-flank,rightBreakPointPos+seqlength-1+flank )))
}




#qual <- readsRanges$qual[2]
decipherMAPQ <- function(qual){
  -10 * log10(as.numeric(qual))
}


#test on the first
#index <- 1
#testR <- readsRanges[1]

#utilities to manage cigar
cigarToVector <- function(cigar){
  outVect <- c()
  chars=c("M|N|D|I|S|H|P|X|=")
  while(cigar != "" ){
    pos <- gregexpr(chars, cigar)[[1]][1]
    char <- substr(cigar,pos,pos)
    actual <- substr(cigar,1,pos)
    number <- as.numeric(substr(cigar,1,pos-1))
    if ( pos+1 <= nchar(cigar)){
      rest <- substr(cigar,pos+1, nchar(cigar))
    }else{
      rest <- ""
    }
    outVect <- c(outVect,char)
    number <- number - 1
    if ( number > 0){
      cigar <- paste(number,char,rest,sep="")
    }else{
      cigar <- rest
    }      
  }
  return(outVect)
}

vectorToCigar <- function(cigarVect){
  cigarLetters <- NULL
  cigarNumbers <- NULL
  while(length(cigarVect) > 0){
    if ( !is.null(cigarLetters) && cigarLetters[length(cigarLetters)] == cigarVect[1] ){
      cigarNumbers[length(cigarNumbers)] <- cigarNumbers[length(cigarNumbers)] + 1
      #cigarLetters[length(cigarLetters)] <- cigarVect[1]
      #cigarVect <-  cigarVect[2:length(cigarVect)]
      cigarVect <-  tail(cigarVect,length(cigarVect)-1)
    }else{
      cigarLetters <- c(cigarLetters,cigarVect[1])
      cigarNumbers <- c(cigarNumbers,1)
      #cigarVect <- cigarVect[2:length(cigarVect)]
      cigarVect <-  tail(cigarVect,length(cigarVect)-1)
    }
  }
  outCigar <- ""
  for(i in c(1:length(cigarLetters))){
    outCigar <- paste(outCigar,as.character(cigarNumbers[i]),cigarLetters[i],sep="")
  }  
  return(outCigar)
}



analyseRead <- function(i,readsRanges,goodRanges,flank){ 
  #read current read
  testR <- readsRanges[i]
  
  #decript BP code
  testRcoords <- fromBPtodRanger(seqnames(testR),width(ranges(testR)))
  
  if (length(which(testRcoords[1] == goodRanges[,1] & testRcoords[3] == goodRanges[,3] &
    goodRanges[,2] - flank <= testRcoords[2] & goodRanges[,2] + flank >= testRcoords[2] &
    goodRanges[,4] - flank <= testRcoords[4] & goodRanges[,4] + flank >= testRcoords[4])) != 1)
  {
    #cat(i,": Cigar ",actualCigar," not yet supported in read ",testR$qname,sep="")
    return(GRanges())
  }
  
  #set common information
  cigar <- elementMetadata(testR)[1,"cigar"]
  #cigar <- testR$cigar
  cigarVect <- cigarToVector(cigar) 
  seq <-  elementMetadata(testR)[1,"seq"]
  #seq <- testR$seq
  qual <- elementMetadata(testR)[1,"qual"]                                 
  #qual <- testR$qual
  mapq <- elementMetadata(testR)[1,"mapq"]
  #mapq <- testR$mapq
  qname <- elementMetadata(testR)[1,"qname"]
  #qname <- testR$qname
  flag <- elementMetadata(testR)[1,"flag"]
  #flag <- testR$flag
  strand = strand(testR) 
  mrnm <- "*" 
  mpos <- 0
  isize <- 0
  
  #decript breakPointer coords
  bed<-fromBPtoBedLarge(seqnames(testR),width(ranges(testR)),flank=20)
  
  #ignore chrX (or chr23) (check if chr24 is inserted!)
  if (bed[1,1] >= 23 | bed[2,1] >= 23){
    return(GRanges())
  }
  
  #build left part of the string    
  start <- start(ranges(testR))
  SeqPosition <- 1
  RefPosition <- 1
  CigarPos <- 1
  leftSeq <- ""
  leftQual <- ""
  leftCigarVect <- c()
  
  
  
  while( (start + RefPosition) <= width(ranges(testR)) && CigarPos <= length(cigarVect)){
    
    actualCigar <- cigarVect[CigarPos]
    leftCigarVect <- c(leftCigarVect,actualCigar)
    if (actualCigar == "M"){
      actualChar <- substr(seq,SeqPosition,SeqPosition)
      leftSeq <- paste(leftSeq,actualChar,sep="")
      actualQual <- substr(qual,SeqPosition,SeqPosition)
      leftQual <- paste(leftQual,actualQual,sep="")  
      RefPosition <- RefPosition + 1
      SeqPosition <- SeqPosition + 1    
      CigarPos <- CigarPos + 1
    }
    if (actualCigar == "D"){
      CigarPos <- CigarPos + 1
      RefPosition <- RefPosition + 1
      #SeqPosition <- SeqPosition + 1
    }
    if (actualCigar == "I"){
      actualChar <- substr(seq,SeqPosition,SeqPosition)
      leftSeq <- paste(leftSeq,actualChar,sep="")
      actualQual <- substr(qual,SeqPosition,SeqPosition)
      leftQual <- paste(leftQual,actualQual,sep="") 
      SeqPosition <- SeqPosition + 1     
      CigarPos <- CigarPos + 1
    }
    if (actualCigar == "S"){
      actualChar <- substr(seq,SeqPosition,SeqPosition)
      leftSeq <- paste(leftSeq,actualChar,sep="")
      actualQual <- substr(qual,SeqPosition,SeqPosition)
      leftQual <- paste(leftQual,actualQual,sep="")  
      RefPosition <- RefPosition + 1
      SeqPosition <- SeqPosition + 1    
      CigarPos <- CigarPos + 1
    }
    if (actualCigar == "H"){
      RefPosition <- RefPosition + 1
      CigarPos <- CigarPos + 1
    }
    if (actualCigar == "N"){
      RefPosition <- RefPosition + 1
      CigarPos <- CigarPos + 1
    }
    if (!actualCigar %in% c("M","D","I","S","H","N")){
      cat(i,": Cigar ",actualCigar," not yet supported in read ",testR$qname,sep="")
      return(GRanges())
    }
    
  }
  if (is.null(leftCigarVect)){return(GRanges())}
  leftCigar <- vectorToCigar(leftCigarVect)
  
  #build right part of the string
  #continue to parse cigar 
  rightSeq <- ""
  rightQual <- ""
  rightCigarVect <- c()
  
  #continue until cigar is consumed
  while( CigarPos <= length(cigarVect)){
    
    actualCigar <- cigarVect[CigarPos]
    rightCigarVect <- c(rightCigarVect,actualCigar)
    if (actualCigar == "M"){
      actualChar <- substr(seq,SeqPosition,SeqPosition)
      rightSeq <- paste(rightSeq,actualChar,sep="")
      actualQual <- substr(qual,SeqPosition,SeqPosition)
      rightQual <- paste(rightQual,actualQual,sep="")  
      RefPosition <- RefPosition + 1
      SeqPosition <- SeqPosition + 1    
      CigarPos <- CigarPos + 1
    }
    if (actualCigar == "D"){
      CigarPos <- CigarPos + 1
      RefPosition <- RefPosition + 1
      #SeqPosition <- SeqPosition + 1
    }
    if (actualCigar == "I"){
      actualChar <- substr(seq,SeqPosition,SeqPosition)
      rightSeq <- paste(rightSeq,actualChar,sep="")
      actualQual <- substr(qual,SeqPosition,SeqPosition)
      rightQual <- paste(rightQual,actualQual,sep="") 
      SeqPosition <- SeqPosition + 1     
      CigarPos <- CigarPos + 1
    }
    if (actualCigar == "S"){
      actualChar <- substr(seq,SeqPosition,SeqPosition)
      leftSeq <- paste(leftSeq,actualChar,sep="")
      actualQual <- substr(qual,SeqPosition,SeqPosition)
      leftQual <- paste(leftQual,actualQual,sep="")  
      RefPosition <- RefPosition + 1
      SeqPosition <- SeqPosition + 1    
      CigarPos <- CigarPos + 1
    }
    if (actualCigar == "H"){
      RefPosition <- RefPosition + 1
      CigarPos <- CigarPos + 1
    }
    if (actualCigar == "N"){
      RefPosition <- RefPosition + 1
      CigarPos <- CigarPos + 1
    }
    if (!actualCigar %in% c("M","D","I","S","H","N")){
      cat(i,": Cigar ",actualCigar," not yet supported in read ",testR$qname,sep="")
      return(GRanges())
    }
  }
  if (is.null(rightCigarVect)){return(GRanges())}
  rightCigar <- vectorToCigar(rightCigarVect)
  
  #retrive reference genome for both breakpoints
  leftReference <- getSeq(Hsapiens, paste("chr",bed[1,1],sep="") , bed[1,2], bed[1,3],as.character=F)
  rightReference <- getSeq(Hsapiens, paste("chr",bed[2,1],sep="") , bed[2,2], bed[2,3],as.character=F)
  
  #find where left query align
  leftQuery <- DNAString(leftSeq)
  alignementL <-pairwiseAlignment(leftQuery,leftReference, type="overlap")
  alignementL.Reverse <-pairwiseAlignment(reverseComplement(leftQuery),leftReference, type="overlap")  
  alignementR <-pairwiseAlignment(leftQuery,rightReference, type="overlap")
  alignementR.Reverse <-pairwiseAlignment(reverseComplement(leftQuery),rightReference, type="overlap")
  scores <- c(score(alignementL), score(alignementL.Reverse), score(alignementR),  score(alignementR.Reverse))
  best <- which.max(scores)
  if ( length(which(scores == scores[best])) > 1 ){ 
    cat(i,": left query ",as.character(leftQuery)," has multiple best (",qname,")\n" )
    leftRead <- GRanges() #return(GRanges())
  }else{
    
    if (best == 1){
      leftStart <- bed[1,2] + start(Views(alignementL)) - 1
      leftRange <- IRanges(start=leftStart,width=nchar(leftSeq))
      leftSeqnames <- bed[1,1]
    }
    if (best == 2){
      leftStart <- bed[1,2] + start(Views(alignementL.Reverse)) - 1
      leftRange <- IRanges(start=leftStart,width=nchar(leftSeq))
      leftSeqnames <- bed[1,1]        
      
      #reverse string and cigar and quality?
      leftSeq <- as.character(reverseComplement(leftQuery))
      leftCigar <- vectorToCigar(reverse(leftCigarVect))
      leftQual <- reverse(leftQual)
    }
    if (best == 3){
      leftStart <- bed[2,2] + start(Views(alignementR)) - 1
      leftRange <- IRanges(start=leftStart,width=nchar(leftSeq))
      leftSeqnames <- bed[2,1]
    }  
    if (best == 4){
      leftStart <- bed[2,2] + start(Views(alignementR.Reverse)) - 1
      leftRange <- IRanges(start=leftStart,width=nchar(leftSeq))
      leftSeqnames <- bed[2,1]        
      
      #reverse string and cigar and quality?
      leftSeq <- as.character(reverseComplement(leftQuery))
      leftCigar <- vectorToCigar(reverse(leftCigarVect))
      leftQual <- reverse(leftQual)
    }
    
    #create left read
    leftRead <- GRanges(seqnames=leftSeqnames, ranges=leftRange, strand=strand, 
                        qname=paste(qname,"_L",sep=""),
                        flag=flag,
                        mapq=mapq,
                        cigar=leftCigar,
                        mrnm=mrnm,
                        mpos=mpos,
                        isize=isize,
                        seq=leftSeq,
                        qual=leftQual )
  }
  #find where right query align
  rightQuery <- DNAString(rightSeq)
  alignementL <- pairwiseAlignment(rightQuery,leftReference, type="overlap")
  alignementL.Reverse <-pairwiseAlignment(reverseComplement(rightQuery),leftReference, type="overlap")  
  alignementR <-pairwiseAlignment(rightQuery,rightReference, type="overlap")
  alignementR.Reverse <-pairwiseAlignment(reverseComplement(rightQuery),rightReference, type="overlap")
  
  scoresR <- c(score(alignementL), score(alignementL.Reverse), score(alignementR),  score(alignementR.Reverse))
  bestR <- which.max(scoresR)
  if ( length(which(scoresR == scoresR[bestR])) > 1 ){ 
    cat(i,": right query ",as.character(rightQuery)," has multiple best (",qname,")\n" )
    #return(GRanges())
    rightRead <- GRanges()
  }else{
    
    best <- which.max(c(score(alignementL), score(alignementL.Reverse), score(alignementR),  score(alignementR.Reverse)))
    
    if (best == 1){
      rightStart <- bed[1,2] + start(Views(alignementL)) - 1
      rightRange <- IRanges(start=rightStart,width=nchar(rightSeq))
      rightSeqnames <- bed[1,1]
    }
    if (best == 2){
      rightStart <- bed[1,2] + start(Views(alignementL.Reverse)) - 1
      rightRange <- IRanges(start=rightStart,width=nchar(rightSeq))
      rightSeqnames <- bed[1,1]        
      
      #reverse string and cigar and quality?
      rightSeq <- as.character(reverseComplement(rightQuery))
      rightCigar <- vectorToCigar(reverse(rightCigarVect))
      rightQual <- reverse(rightQual)
    }
    if (best == 3){
      rightStart <- bed[2,2] + start(Views(alignementR)) - 1
      rightRange <- IRanges(start=rightStart,width=nchar(rightSeq))
      rightSeqnames <- bed[2,1]
    }  
    if (best == 4){
      rightStart <- bed[2,2] + start(Views(alignementR.Reverse)) - 1
      rightRange <- IRanges(start=rightStart,width=nchar(rightSeq))
      rightSeqnames <- bed[2,1]        
      
      #reverse string and cigar and quality?
      rightSeq <- as.character(reverseComplement(rightQuery))
      rightCigar <- vectorToCigar(reverse(rightCigarVect))
      rightQual <- reverse(rightQual)
    }
    
    #create left read
    rightRead <- GRanges(seqnames=rightSeqnames, ranges=rightRange, strand=strand, 
                         qname=paste(qname,"_R",sep=""),
                         flag=flag,
                         mapq=mapq,
                         cigar=rightCigar,
                         mrnm=mrnm,
                         mpos=mpos,
                         isize=isize,
                         seq=rightSeq,
                         qual=rightQual )  
  }
  
  return(c(leftRead,rightRead))
  
}


analyseRead.tryCatch <- function(i,readsRanges,goodRanges,flank){
  tryCatch(analyseRead(i,readsRanges,goodRanges,flank), 
           error = function(e){ 
             #read current read
             testR <- readsRanges[i]
             cigar <- elementMetadata(testR)[1,"cigar"]
             qname <- elementMetadata(testR)[1,"qname"]
             cat("Error on read ",i," cigar is ",cigar," qname is ",qname,"\n"  )
                                }
           )
}



######################################################################
#
# Compute ctm.global
#
######################################################################
#ctm.global.data <- computeCTMglobal(betaTableDeletions)  
computeCTMglobal <- function(betaTableDeletions){
  
  dataOut <- data.frame(row.names=1)
  dataOut$median <- NA
  dataOut$min <- NA
  dataOut$max <- NA
  dataOut$total.zones <- NA
  dataOut$used.zones <- NA
  
  if (nrow(betaTableDeletions) == 0){  return(dataOut) }
  
  #order by beta
  betaTableDeletions <- betaTableDeletions[order(betaTableDeletions$adm.local),]
  
  
  #distance <- dist(betaTableDeletions$beta,"eucl")  
  #hc <- hclust(distance, "single")
  #plot(hc,cex=0.5)
  #groups <- cutree(hc,2)
  #rect.hclust(hc,k=2,border="red4")
  #km <- kmeans(betaTableDeletions$adm.local,2)
  #min(betaTableDeletions$beta[which(km$cluster==2)])
  #max(betaTableDeletions$beta[which(km$cluster==2)])
  
  #iterate until error cannot explain the difference
  found <- F
  geneId <- 1
  adms <-  betaTableDeletions$adm.local[geneId]
  nsnps <- betaTableDeletions$nsnp[geneId]
  cov <- betaTableDeletions$cov[geneId]
  
  idxError <- which(errorTable$n.info.snps <= nsnps & errorTable$mean.cov <= cov)
  if (length(idxError) == 0 ){idxError <- 1}    
  local.errors <- errorTable$adm.estimation.error[max(idxError)]
    
  geneId <- 2
  while( !found && geneId <= nrow(betaTableDeletions)){
    adm_actual <-   betaTableDeletions$adm.local[geneId]
    nsnps_actual <- betaTableDeletions$nsnp[geneId]
    cov_actual <- betaTableDeletions$cov[geneId]    

    idxError_actual <- which(errorTable$n.info.snps <= nsnps_actual & errorTable$mean.cov <= cov_actual)
    if (length(idxError_actual) == 0 ){idxError_actual <- 1}    
    local.error_actual <- errorTable$adm.estimation.error[max(idxError_actual)]
    
    
    if ( (round(adms[geneId-1] + local.errors[geneId-1],3)) >=  (round(adm_actual-local.error_actual,3))  ){
      adms<-c(adms,adm_actual)
      local.errors<-c(local.errors,local.error_actual)
      geneId <- geneId + 1
    }else{
      found <- T
    }
    
  }
  
  if (length(adms) > 1){
  #ctm median, min, max, n zones, used all zones?
    dataOut$median <- median(adms)    
    dataOut$min <- min(adms)
    dataOut$max <- max(adms)
    dataOut$total.zones <- nrow(betaTableDeletions)
    dataOut$used.zones <- length(adms)
  }else{
    dataOut$median <- adms    
    dataOut$min <- max(0,adms - local.errors)
    dataOut$max <- min(1,adms + local.errors)
    dataOut$total.zones <- nrow(betaTableDeletions)
    dataOut$used.zones <- 1
    
  }
  return(dataOut)
  
  
}



#takes the list of beta tables with gene based and segseq based betas and compute ctm global
computeCTMglobal.perSample <- function(betaTable,deletionsLog2Levels,min_nsnps,min_cov,getReport=F,reportFile="",nRow.report=0){
  #cat(i,"\n")
  

  #check if betaTable 
  if ( length(betaTable) == 0 || nrow(betaTable) == 0 ){
    cat("Warning! File ",betaTables_files[i]," contains an empty table!\n",sep="")
    return(NULL)
  }
  
    
  #save some global information
  globalAdmTable <- data.frame(row.names=betaTable$sample[1],stringsAsFactors=F)  

  globalAdmTable$sample <- betaTable$sample[1]
  globalAdmTable$n.segments <- nrow(betaTable)
  globalAdmTable$n.segments.NA <- length(which(is.na(betaTable$beta)))
  globalAdmTable$n.segments.good <- length(which(!is.na(betaTable$beta) & (betaTable$nsnp >= min_nsnps) & (betaTable$cov >= min_cov)))
  globalAdmTable$n.segments.deleted <- length(which(!is.na(betaTable$beta) & (betaTable$nsnp >= min_nsnps) & (betaTable$cov >= min_cov) &  
    betaTable$log2 >= deletionsLog2Levels[1] & betaTable$log2 <= deletionsLog2Levels[2] ))
  
  betaTableDeletions <- betaTable[which(!is.na(betaTable$beta) & (betaTable$nsnp >= min_nsnps) & (betaTable$cov >= min_cov) &  
    betaTable$log2 >= deletionsLog2Levels[1] & betaTable$log2 <= deletionsLog2Levels[2] ),]
  
  betaTableDeletions$adm.local <- betaTableDeletions$beta / ( 2 - betaTableDeletions$beta )
  
  if(getReport){    
#    reportMut <-  betaTableDeletions[order(betaTableDeletions$beta),]
#    reportMut <- reportMut[1:nRow.report,]
#    reportMut$cn <- rep(1,nrow(reportMut))
#    write.table(reportMut,file=reportFile,append=T,quote=F,sep="\t",row.names=F,col.names=F)
  }
  
  ctm.global.data <- computeCTMglobal(betaTableDeletions)  

  
  globalAdmTable$median <- ctm.global.data$median
  globalAdmTable$min <- ctm.global.data$min
  globalAdmTable$max <- ctm.global.data$max
  globalAdmTable$total.zones <- ctm.global.data$total.zones
  globalAdmTable$used.zones <- ctm.global.data$used.zones
  
  return(globalAdmTable)
}
#plot(genesTable$AFmean,genesTable$ctm.local.genes )
#hist(betaTable$SegSeq.log2ratio.1,breaks=seq(min(betaTable$SegSeq.log2ratio.1,na.rm=T),max(betaTable$SegSeq.log2ratio.1,na.rm=T),0.005))

#use also cn = 3
computeCTMglobal.perSample.cns.1.3 <- function(betaTable,deletionsLog2Levels,min_nsnps_cn1,min_nsnps_cn3,min_cov,getReport=F,reportFile="",nRow.report=0){
  #cat(i,"\n")
    
  #check if betaTable 
  if ( length(betaTable) == 0 || nrow(betaTable) == 0 ){
    cat("Warning! File ",betaTables_files[i]," contains an empty table!\n",sep="")
    return(NULL)
  }
  
  
  #save some global information
  globalAdmTable <- data.frame(row.names=betaTable$sample[1],stringsAsFactors=F)  
  
  globalAdmTable$sample <- betaTable$sample[1]
  globalAdmTable$n.segments <- nrow(betaTable)
  globalAdmTable$n.segments.NA <- length(which(is.na(betaTable$beta)))
  globalAdmTable$n.segments.good <- "not used"
  
  globalAdmTable$n.segments.mutated <- 
    length(which(
      !is.na(betaTable$beta) & 
      (betaTable$cov >= min_cov) &  
    ( ( betaTable$nsnp >= min_nsnps_cn1 & betaTable$log2 >= deletionsLog2Levels[1] & betaTable$log2 <= deletionsLog2Levels[2] ) |
    ( ( betaTable$nsnp >= min_nsnps_cn3 & betaTable$log2 >= deletionsLog2Levels[3] & betaTable$log2 <= deletionsLog2Levels[4] ) )
    )))
  
  betaTableMutations <- betaTable[
    which(
      !is.na(betaTable$beta) & 
        (betaTable$cov >= min_cov) &  
        ( ( betaTable$nsnp >= min_nsnps_cn1 & betaTable$log2 >= deletionsLog2Levels[1] & betaTable$log2 <= deletionsLog2Levels[2] ) |
        ( ( betaTable$nsnp >= min_nsnps_cn3 & betaTable$log2 >= deletionsLog2Levels[3] & betaTable$log2 <= deletionsLog2Levels[4] ) )
        )),]
  
  
  #correct cn3 data  
  cn1idx <- which(betaTableMutations$log2 >= deletionsLog2Levels[1] & betaTableMutations$log2 <= deletionsLog2Levels[2])
  cn3idx <- which(betaTableMutations$log2 >= deletionsLog2Levels[3] & betaTableMutations$log2 <= deletionsLog2Levels[4])
  
  #only beta > 0.66 is good
  #the other case are cn = 3 where the non duplicated allele is (subclonally) deleted
  #cn3idx <- cn3idx[which(betaTableMutations$beta[cn3idx] > ( 2/3 + 0.1)) ]
  cn3idx <- cn3idx[which(betaTableMutations$beta[cn3idx] > ( 0.72)) ]
  
  #correct beta
  cn3betaCorr <-   1 - 3 * (1 - betaTableMutations$beta[cn3idx])
  betaTableMutations$beta[cn3idx] <- cn3betaCorr
  
  betaTableMutations$adm.local <- rep(NA,nrow(betaTableMutations))
  
  betaTableMutations$adm.local[cn1idx] <- betaTableMutations$beta[cn1idx] / ( 2 - betaTableMutations$beta[cn1idx] )
  betaTableMutations$adm.local[cn3idx] <- ( 3 *  betaTableMutations$beta[cn3idx] )  / ( betaTableMutations$beta[cn3idx] +2 )
  
  betaTableMutations <- betaTableMutations[c(cn1idx,cn3idx),]
  

  
  #compute adm.global
  ctm.global.data <- computeCTMglobal(betaTableMutations)
  
  
  globalAdmTable$median <- ctm.global.data$median
  globalAdmTable$min <- ctm.global.data$min
  globalAdmTable$max <- ctm.global.data$max
  globalAdmTable$total.zones <- ctm.global.data$total.zones
  globalAdmTable$used.zones <- ctm.global.data$used.zones
  
  if(getReport){    
    #    reportMut <-  betaTableMutations[order(betaTableMutations$beta),]
    #    reportMut <- reportMut[1:nRow.report,]
    #    reportMut$cn <- rep(3,nrow(reportMut))
    #    write.table(reportMut,file=reportFile,append=T,quote=F,sep="\t",row.names=F,col.names=F)
    
    
    par(mfrow=c(1,2),oma=c(0,0,3,0),mar=c(5,4,1,2))
    plot(betaTable$log2,betaTable$beta, pch=20,ylim=c(0,1),ylab="beta",xlab="log2",xlim=c(-2,2),main="All SCNA")
    abline(v=log2(seq(1,3)/2),col="blue4")
    plot(betaTableMutations$log2,betaTableMutations$beta, pch=20,ylim=c(0,1),ylab="beta",xlab="log2"  ,xlim=c(-2,2),main="Good SCNA")
    abline(v=log2(seq(1,3)/2),col="blue4")

    betaAdm <- (2*ctm.global.data$median)/(1+ctm.global.data$median)
    betaAdm.min <- (2*ctm.global.data$min)/(1+ctm.global.data$min)
    betaAdm.max <- (2*ctm.global.data$max)/(1+ctm.global.data$max)
    abline(h=c(betaAdm.min,betaAdm,betaAdm.max),col="blue4")
    
    mtext(text=betaTable$sample[1],side=3,outer=T,font=2,cex=1.5,line=1)
  }
  
  return(globalAdmTable)
  
}



########################
## new design based on sd model

computeCTMglobal.SDmodel <- function(betaTableDeletions,errorTable,roundDec=4){
  
  dataOut <- data.frame(row.names=1)
  dataOut$median <- NA
  dataOut$min <- NA
  dataOut$max <- NA
  dataOut$total.zones <- NA
  dataOut$used.zones <- NA
  dataOut$mean <- NA
  dataOut$SD <- NA
  
  
  if (nrow(betaTableDeletions) == 0){  return(dataOut) }
  
  #order by beta
  betaTableDeletions <- betaTableDeletions[order(betaTableDeletions$beta),]
  
  
  #distance <- dist(betaTableDeletions$beta,"eucl")  
  #hc <- hclust(distance, "single")
  #plot(hc,cex=0.5)
  #groups <- cutree(hc,2)
  #rect.hclust(hc,k=2,border="red4")
  #km <- kmeans(betaTableDeletions$adm.local,2)
  #min(betaTableDeletions$beta[which(km$cluster==2)])
  #max(betaTableDeletions$beta[which(km$cluster==2)])
  
  #iterate until error cannot explain the difference
  found <- F
  geneId <- 1
  betas <-  betaTableDeletions$beta[geneId]
  nsnps <- betaTableDeletions$nsnp[geneId]
  cov <- betaTableDeletions$cov[geneId]
  
  idxError <- which(errorTable$n.info.snps <= nsnps & errorTable$mean.cov <= cov)
  if (length(idxError) == 0 ){idxError <- 1}    
  local.errors <- errorTable$adm.estimation.error[max(idxError)]
  mean.errors <- errorTable$mean.adm.estimation.error[max(idxError)]
  SD.errors <- errorTable$SD.adm.estimation.error[max(idxError)]
  
  geneId <- 2
  while( !found && geneId <= nrow(betaTableDeletions)){
    beta_actual <-   betaTableDeletions$beta[geneId]
    nsnps_actual <- betaTableDeletions$nsnp[geneId]
    cov_actual <- betaTableDeletions$cov[geneId]    
    
    idxError_actual <- which(errorTable$n.info.snps <= nsnps_actual & errorTable$mean.cov <= cov_actual)
    if (length(idxError_actual) == 0 ){idxError_actual <- 1}    
    local.error_actual <- errorTable$adm.estimation.error[max(idxError_actual)]
    mean.error_actual <- errorTable$mean.adm.estimation.error[max(idxError_actual)]
    SD.error_actual <- errorTable$SD.adm.estimation.error[max(idxError_actual)]
    
    
    #if ( (round(betas[geneId-1] + local.errors[geneId-1],roundDec)) >=  (round(beta_actual-local.error_actual,roundDec))  ){
    if ( (betas[geneId-1] + local.errors[geneId-1]) >=  (beta_actual-local.error_actual)  ){
      betas<-c(betas,beta_actual)
      local.errors<-c(local.errors,local.error_actual)
      mean.errors<-c(mean.errors,mean.error_actual)
      SD.errors<-c(SD.errors,SD.error_actual)
      geneId <- geneId + 1
    }else{
      found <- T
    }
    
  }
  
  
  if (length(betas) > 1){
    #ctm median, min, max, n zones, used all zones?
    dataOut$median <- median(betas)    
    dataOut$min <- min(betas)
    dataOut$max <- max(betas)
    dataOut$total.zones <- nrow(betaTableDeletions)
    dataOut$used.zones <- length(betas)
    dataOut$mean <- mean(betas)
    dataOut$SD <- sd(betas)
  }else{
    dataOut$median <- betas    
    dataOut$min <- max(0,betas - local.errors)
    dataOut$max <- min(1,betas + local.errors)
    dataOut$total.zones <- nrow(betaTableDeletions)
    dataOut$used.zones <- 1
    dataOut$mean <- betas
    dataOut$SD <- SD.errors
  
  }
  return(dataOut)
  
  
}

computeCTMglobal.perSample.SDmodel <- function(betaTable,deletionsLog2Levels,min_nsnps,min_cov,errorTable,getReport=F,reportFile="",alpha=0.01){
  #cat(i,"\n")
  
  #check if betaTable 
  if ( length(betaTable) == 0 || nrow(betaTable) == 0 ){
    cat("Warning! File ",betaTables_files[i]," contains an empty table!\n",sep="")
    return(NULL)
  }
  
  
  #save some global information
  globalAdmTable <- data.frame(row.names=betaTable$sample[1],stringsAsFactors=F)  
  
  globalAdmTable$sample <- betaTable$sample[1]
  globalAdmTable$n.segments <- nrow(betaTable)
  globalAdmTable$n.segments.NA <- length(which(is.na(betaTable$beta)))
  globalAdmTable$n.segments.good <- "not used"
  
  globalAdmTable$n.segments.mutated <- 
    length(which(
        !is.na(betaTable$beta) & 
        betaTable$cov >= min_cov &  
        ( ( betaTable$nsnp >= min_nsnps & betaTable$log2 >= deletionsLog2Levels[1] & betaTable$log2 <= deletionsLog2Levels[2] )
        )))
  
  betaTableDeletions <- betaTable[
    which(
      !is.na(betaTable$beta) & 
        betaTable$cov >= min_cov &  
        ( ( betaTable$nsnp >= min_nsnps & betaTable$log2 >= deletionsLog2Levels[1] & betaTable$log2 <= deletionsLog2Levels[2] )
        )),]
  
  
  
  
  #compute adm.global
  ctm.global.data <- computeCTMglobal.SDmodel(betaTableDeletions=betaTableDeletions,errorTable=errorTable)
  
  beta <- c(ctm.global.data$mean,ctm.global.data$SD)
  fake <- c(1,1)
  convertExpression <- expression(beta / (2 - beta))
  DF <- cbind(beta,fake)
  res <- propagate(expr = convertExpression, data = DF, type = "stat", do.sim = TRUE, do.perm=TRUE, verbose = F,plot=F,alpha=alpha,) 
    
  globalAdmTable$mean <- res$summary$Prop[1]
  globalAdmTable$SD <- res$summary$Prop[2]
  globalAdmTable$Conf.lower <- res$summary$Prop[5]
  globalAdmTable$Conf.upper <- res$summary$Prop[6]
  
  if(getReport){    
    #    reportMut <-  betaTableMutations[order(betaTableMutations$beta),]
    #    reportMut <- reportMut[1:nRow.report,]
    #    reportMut$cn <- rep(3,nrow(reportMut))
    #    write.table(reportMut,file=reportFile,append=T,quote=F,sep="\t",row.names=F,col.names=F)
    
    
    par(mfrow=c(1,2),oma=c(0,0,3,0),mar=c(5,4,1,2))
    plot(betaTable$log2,betaTable$beta, pch=20,ylim=c(0,1),ylab="beta",xlab="log2",xlim=c(-2,2),main="All SCNA")
    abline(v=log2(seq(1,3)/2),col="blue4")
    abline(h=c(ctm.global.data$mean,ctm.global.data$mean-ctm.global.data$SD,ctm.global.data$mean+ctm.global.data$SD),col="blue4")
    plot( betaTableDeletions$log2,betaTableDeletions$beta, pch=20,ylim=c(0,1),ylab="beta",xlab="log2"  ,xlim=c(-2,2),main="Good SCNA")
    abline(v=log2(seq(1,3)/2),col="blue4")
    abline(h=c(ctm.global.data$mean,ctm.global.data$mean-ctm.global.data$SD,ctm.global.data$mean+ctm.global.data$SD),col="blue4")
    
    
    
    mtext(text=betaTable$sample[1],side=3,outer=T,font=2,cex=1.5,line=1)
  }
  
  return(globalAdmTable)
  
}


########################
## bettter report
## gives only the cluster and delay the real adm global estimation

clusterRelevanBetas <- function(betaTableDeletions,errorTable,roundDec=4){
  
  if (nrow(betaTableDeletions) == 0){  return(NULL) }
  #order by beta
  betaTableDeletions <- betaTableDeletions[order(betaTableDeletions$beta),]
  
  #iterate until error cannot explain the difference
  found <- F
  geneId <- 1
  betas <-  betaTableDeletions$beta[geneId]
  nsnps <- betaTableDeletions$nsnp[geneId]
  cov <- betaTableDeletions$cov[geneId]
  
  idxError <- which(errorTable$n.info.snps <= nsnps & errorTable$mean.cov <= cov)
  if (length(idxError) == 0 ){idxError <- 1}    
  local.errors <- errorTable$adm.estimation.error[max(idxError)]
  
  geneId <- 2
  while( !found && geneId <= nrow(betaTableDeletions)){
    beta_actual <-   betaTableDeletions$beta[geneId]
    nsnps_actual <- betaTableDeletions$nsnp[geneId]
    cov_actual <- betaTableDeletions$cov[geneId]    
    
    idxError_actual <- which(errorTable$n.info.snps <= nsnps_actual & errorTable$mean.cov <= cov_actual)
    if (length(idxError_actual) == 0 ){idxError_actual <- 1}    
    local.error_actual <- errorTable$adm.estimation.error[max(idxError_actual)]
    
    
    #if ( (round(betas[geneId-1] + local.errors[geneId-1],roundDec)) >=  (round(beta_actual-local.error_actual,roundDec))  ){
    if ( (betas[geneId-1] + local.errors[geneId-1]) >=  (beta_actual-local.error_actual)  ){
    #if ( (betas[geneId-1] + local.errors[geneId-1]) >=  (beta_actual)    &&   betas[geneId-1] >=  (beta_actual -local.error_actual) ){
      betas<-c(betas,beta_actual)
      local.errors<-c(local.errors,local.error_actual)
      geneId <- geneId + 1
    }else{
      found <- T
    }
    
  }
  
  
  #plot(betaTableDeletions$log2, betaTableDeletions$beta, pch=20 )
  #hist(betas)
  #plot(ecdf(betas))
  #quantile(ecdf(betas),probs=0.05)
  #quantile(ecdf(betas),probs=0.95)
  #mean(betas)
  
  if (length(betas) > 1){
    goodBetaTable <- betaTableDeletions[1:length(betas),]
    goodBetaTable$errorBeta <- local.errors   
  }else{
    goodBetaTable <- betaTableDeletions[1,]
    goodBetaTable$errorBeta <- local.errors   
  }
  return(goodBetaTable)  
}


getRelevantBetasDels_AdmGlobal <- function(betaTable,deletionsLog2Levels,min_nsnps,min_cov,errorTable){
  #check if betaTable 
  if ( length(betaTable) == 0 || nrow(betaTable) == 0 ){
    cat("Warning! File ",betaTables_files[i]," contains an empty table!\n",sep="")
    return(NULL)
  }
  
  betaTableDeletions <- betaTable[
    which(
      !is.na(betaTable$beta) & 
        betaTable$cov >= min_cov &  
        ( ( betaTable$nsnp >= min_nsnps & betaTable$log2 >= deletionsLog2Levels[1] & betaTable$log2 <= deletionsLog2Levels[2] )
        )),]
  
  goodDeletions <- clusterRelevanBetas(betaTableDeletions,errorTable)
  
  return(goodDeletions)
  
}



getRelevantBetasGains_AdmGlobal <- function(betaTable,deletionsLog2Levels,min_nsnps,min_cov,errorTable){
  #check if betaTable 
  if ( length(betaTable) == 0 || nrow(betaTable) == 0 ){
    cat("Warning! File ",betaTables_files[i]," contains an empty table!\n",sep="")
    return(NULL)
  }
  
  betaTableGains <- betaTable[
    which(
      !is.na(betaTable$beta) & 
        betaTable$cov >= min_cov &  
        ( ( betaTable$nsnp >= min_nsnps & betaTable$log2 >= deletionsLog2Levels[3] & betaTable$log2 <= deletionsLog2Levels[4] )) &
        betaTable$beta >= 0.72
      ),]
  
  goodGains <- clusterRelevanBetas(betaTableGains,errorTable)
  
  return(goodGains)
  
}

######################################################################
#
# Compute clonality and clonality status
#
######################################################################


##################









########################################################
########################################################
#
# Analyze clonality data
#
########################################################
########################################################
getSampleSummary <- function(x){
  #integerCN <- unlist(lapply(x$SegSeq.zone.CN.1,computeIntegerCN,cn_levels))
  tryCatch({
    if(length(unique(x$gene)) < nrow(x) ){
      x <- x[which(!duplicated(x$gene)), ]
    }   
    sampleData <- data.frame(row.names=x$arrayName[1])
    sampleData$bamName <- x$bamName[1]
    sampleData$arrayName <- x$arrayName[1]
    sampleData$adm.global <- x$adm.global.thisZone[1]  
    sampleData$Gleason.ExomeWGS.Centralized <- x$Gleason.ExomeWGS.Centralized[1]
    sampleData$GleasonMajor.ExomeWGS.Centralized <- x$GleasonMajor.ExomeWGS.Centralized[1]
    sampleData$Pathologic.T.stage_T2T3 <- x$Pathologic.T.stage_T2T3[1]
    sampleData$ERG.FISH.status.exome.WGS.plus <- x$ERG.FISH.status.exome.WGS.plus[1]
    sampleData$ngenes <- nrow(x)
    
    x <- x[which(x$bestSeg.nSNPs >= minSNPs & x$bestSeg.cov >= minCov & !(x$isGermline) & !is.na(x$bestSeg.beta) ),]
    
    if( nrow(x) == 0 ){    
      sampleData$nAnalysedGenes <- NA
      sampleData$ngenesLog2 <- NA      
      sampleData$nAnalysedGenesDeleted <- NA
      sampleData$nAnalysedGenesDeletedSubclonal <- NA    
      #geneData$nGenesDel <- NA
      geneData$nGenesDelAn <- NA
      geneData$nGenesDelAnSubclonal <- NA
      return(sampleData)
    }
    
    #sampleData$nAnalysedGenes <- length(which(!is.na(x$betas) & x$nSNPs >= minSNPs & x$cov >= minCov & !(x$isGermline)))
    sampleData$nAnalysedGenes <- nrow(x)
    
    sampleData$CN4p.clo <- length(which(x$bestSeg.CNs > 3 & (x$bestSeg.clonStatus == "clonal")))
    sampleData$CN4p.sub <- length(which(x$bestSeg.CNs > 3 & (x$bestSeg.clonStatus == "subclonal")))
    sampleData$CN4p.unc <- length(which(x$bestSeg.CNs > 3 & (x$bestSeg.clonStatus == "uncertain")))
    
    sampleData$CN3.clo <- length(which(x$bestSeg.CNs == 3 & (x$bestSeg.clonStatus == "clonal")))
    sampleData$CN3.sub <- length(which(x$bestSeg.CNs == 3 & (x$bestSeg.clonStatus == "subclonal")))
    sampleData$CN3.unc <- length(which(x$bestSeg.CNs == 3 & (x$bestSeg.clonStatus == "uncertain")))
    
    sampleData$CN2.clo <- length(which(x$bestSeg.CNs == 2 & (x$bestSeg.clonStatus == "clonal")))                  
    sampleData$CN2.sub <- length(which(x$bestSeg.CNs == 2 & (x$bestSeg.clonStatus == "subclonal")))                  
    sampleData$CN2.unc <- length(which(x$bestSeg.CNs == 2 & (x$bestSeg.clonStatus == "uncertain")))
    
    sampleData$CN1.clo <- length(which(x$bestSeg.CNs == 1 & (x$bestSeg.clonStatus == "clonal")))
    sampleData$CN1.sub <- length(which(x$bestSeg.CNs == 1 & (x$bestSeg.clonStatus == "subclonal")))    
    sampleData$CN1.unc <- length(which(x$bestSeg.CNs == 1 & (x$bestSeg.clonStatus == "uncertain")))
    
    sampleData$CN0.clo <- length(which(x$bestSeg.CNs == 0 & (x$bestSeg.clonStatus == "clonal")))
    sampleData$CN0.sub <- length(which(x$bestSeg.CNs == 0  & (x$bestSeg.clonStatus == "subclonal") ))
    sampleData$CN0.unc <- length(which(x$bestSeg.CNs == 0 & (x$bestSeg.clonStatus == "uncertain")))
  },
    error = function(e){cat("Error on mutation ",regionData$gene," of sample ", regionData$bamName,"\n",sep=""  )}
  )
  return(sampleData)
}



getGeneSummary <- function(x){
  #integerCN <- unlist(lapply(x$SegSeq.zone.CN.1,computeIntegerCN,cn_levels))
  if(length(unique(x$arrayName)) < nrow(x) ){
    x <- x[which(!duplicated(x$gene)), ]
  }   
  geneData <- data.frame(row.names=x$gene[1])
  geneData$gene <- x$gene[1]
  x<-x[which(!x$isGermline & !(is.na(x$bestSeg.beta))),]
  geneData$n.gene.analyzed <- nrow(x)
    
  geneData$CN0 <- length(which(x$bestSeg.CNs==0))
  geneData$CN1 <- length(which(x$bestSeg.CNs==1))
  geneData$CN2 <- length(which(x$bestSeg.CNs==2))
  geneData$CN3 <- length(which(x$bestSeg.CNs==3))
  geneData$CN4p <- length(which(x$bestSeg.CNs>3))
  
  geneData$CN0.clo <- length(which(x$bestSeg.CNs == 0 & (x$bestSeg.clonStatus == "clonal")))
  geneData$CN0.sub <- length(which(x$bestSeg.CNs == 0 & (x$bestSeg.clonStatus == "subclonal")))
  geneData$CN0.unc<- length(which(x$bestSeg.CNs == 0 & (x$bestSeg.clonStatus == "uncertain")))
  
  geneData$CN1.clo <- length(which(x$bestSeg.CNs==1  & (x$bestSeg.clonStatus == "clonal")))
  geneData$CN1.sub <- length(which(x$bestSeg.CNs==1  & (x$bestSeg.clonStatus == "subclonal")))
  geneData$CN1.unc <- length(which(x$bestSeg.CNs==1  & (x$bestSeg.clonStatus == "uncertain")))
  
  geneData$CN2.clo <- length(which(x$bestSeg.CNs==2  & (x$bestSeg.clonStatus == "clonal")))
  geneData$CN2.sub <- length(which(x$bestSeg.CNs==2  & (x$bestSeg.clonStatus == "subclonal")))
  geneData$CN2.unc <- length(which(x$bestSeg.CNs==2  & (x$bestSeg.clonStatus == "uncertain")))
  
  
  geneData$CN3.clo <- length(which(x$bestSeg.CNs==3  & (x$bestSeg.clonStatus == "clonal")))
  geneData$CN3.sub <- length(which(x$bestSeg.CNs==3  & (x$bestSeg.clonStatus == "subclonal")))
  geneData$CN3.unc <- length(which(x$bestSeg.CNs==2  & (x$bestSeg.clonStatus == "uncertain")))
  
  geneData$CN4p.clo <- length(which(x$bestSeg.CNs>3  & (x$bestSeg.clonStatus == "clonal")))
  geneData$CN4p.sub <- length(which(x$bestSeg.CNs>3  & (x$bestSeg.clonStatus == "subclonal")))
  geneData$CN4p.unc <- length(which(x$bestSeg.CNs>3  & (x$bestSeg.clonStatus == "uncertain")))
  
  return(geneData)
}


getChrSummary <- function(x){
  #integerCN <- unlist(lapply(x$SegSeq.zone.CN.1,computeIntegerCN,cn_levels))
  if(length(unique(paste(x$gene,x$arrayName," "))) < nrow(x) ){
    x <- x[which(!duplicated(paste(x$gene,x$arrayName," "))), ]
  }   
  chrData <- data.frame(row.names=x$chr[1])
  chrData$chr <- x$chr[1]
  x<-x[which(!x$isGermline & !(is.na(x$bestSeg.beta))),]
  chrData$n.gene.analyzed <- nrow(x)
  
  chrData$CN0 <- length(which(x$bestSeg.CNs==0))
  chrData$CN1 <- length(which(x$bestSeg.CNs==1))
  chrData$CN2 <- length(which(x$bestSeg.CNs==2))
  chrData$CN3 <- length(which(x$bestSeg.CNs==3))
  chrData$CN4p <- length(which(x$bestSeg.CNs>3))
  
  chrData$CN0.clo <- length(which(x$bestSeg.CNs == 0 & (x$bestSeg.clonStatus == "clonal")))
  chrData$CN0.sub <- length(which(x$bestSeg.CNs == 0 & (x$bestSeg.clonStatus == "subclonal")))
  chrData$CN0.unc <- length(which(x$bestSeg.CNs == 0 & (x$bestSeg.clonStatus == "uncertain")))
  
  chrData$CN1.clo <- length(which(x$bestSeg.CNs==1  & (x$bestSeg.clonStatus == "clonal")))
  chrData$CN1.sub <- length(which(x$bestSeg.CNs==1  & (x$bestSeg.clonStatus == "subclonal")))
  chrData$CN1.unc <- length(which(x$bestSeg.CNs==1  & (x$bestSeg.clonStatus == "uncertain")))
  
  chrData$CN2.clo <- length(which(x$bestSeg.CNs==2  & (x$bestSeg.clonStatus == "clonal")))
  chrData$CN2.sub <- length(which(x$bestSeg.CNs==2  & (x$bestSeg.clonStatus == "subclonal")))
  chrData$CN2.unc <- length(which(x$bestSeg.CNs==2  & (x$bestSeg.clonStatus == "uncertain")))
  
  chrData$CN3.clo <- length(which(x$bestSeg.CNs==3  & (x$bestSeg.clonStatus == "clonal")))
  chrData$CN3.sub <- length(which(x$bestSeg.CNs==3  & (x$bestSeg.clonStatus == "subclonal")))
  chrData$CN3.unc <- length(which(x$bestSeg.CNs==3  & (x$bestSeg.clonStatus == "uncertain")))
  
  chrData$CN4p.clo <- length(which(x$bestSeg.CNs>3  & (x$bestSeg.clonStatus == "clonal")))
  chrData$CN4p.sub <- length(which(x$bestSeg.CNs>3  & (x$bestSeg.clonStatus == "subclonal")))
  chrData$CN4p.unc <- length(which(x$bestSeg.CNs>3  & (x$bestSeg.clonStatus == "uncertain")))
  
  return(chrData)
}





##################################################
##################################################
#
# Create a summary table from Rdata analysis file
# 
##################################################
##################################################


#X<-analysisFiles.list[[1]]
createCNanalysisBetaTable <- function(segSeqAnalysisFile,suffix,n_corePerSample){
  
  #check if file exists
  if(!fileExistsAndAccessible(segSeqAnalysisFile)){
    cat("Error: file ",segSeqAnalysisFile," does not exist or it is not accessible",sep="")
    return(NULL)
  }
  
  
  #load analysis files
  load(segSeqAnalysisFile)
  segSeqAnalysis <- dataOut
  
  
  sampleName <- gsub(suffix,"",basename(segSeqAnalysisFile))
  
  #actualTime <- format(Sys.time(),"%b %d %X" )
  #cat(actualTime," Starting analysis of sample ",sampleName,"\n",sep="")
  
  #load list of gene and seg seq coords  
  listSegSeqAnalysisCoords <- do.call(rbind,lapply(segSeqAnalysis$results,function(x){return(x$mutation)}))
  
  #aa <- lapply(segSeqAnalysis$results,function(x){cat(unlist(x$mutation));  return(x$mutation)})
  
  #segSeqAnalysis$results[[4]]
  
  ## analysisEntry <- segSeqAnalysis$results[[13233]]
  getData <- function(analysisEntry, sampleName){
    
    segInfo <- data.frame(row.names=1)        
    segInfo$sample <- sampleName
    
    #this line need to be improved
    
    
    if (is.null(analysisEntry)){
      segInfo$chr <- NA
      segInfo$start <- NA
      segInfo$end <- NA
      segInfo$log2 <- NA
    }else{
      segInfo$chr <- analysisEntry$mutation[[1]]
      segInfo$start <- analysisEntry$mutation[[2]]
      segInfo$end <- analysisEntry$mutation[[3]]
      segInfo$log2 <- analysisEntry$mutation[[4]]
    }
    
    
    
    if (is.null(analysisEntry$computing)){
      segInfo$beta <- NA
      segInfo$nref <- NA
      segInfo$nsnp <- NA
      segInfo$cov <- NA
      segInfo$sd <- NA
      segInfo$AFmean <- NA
      segInfo$AFsd <- NA
      segInfo$pval <- NA
    }else{
      computing <- analysisEntry$computing
      segInfo$beta <- round(computing$pso$par[1],3)
      segInfo$nref <- round(computing$pso$par[2],3)
      segInfo$nsnp <- length(computing$tn$ref)
      segInfo$cov <- round(computing$cov,3)
      segInfo$sd <- round(computing$sd,3)
      covs <- computing$tn$ref + computing$tn$alt
      num <- pmin(computing$tn$alt,computing$tn$ref)
      segInfo$AFmean <- round(mean(num/covs),3)
      segInfo$AFsd <- round(sd(num/covs),3)
      computedCumulative <- stepfun(x=c(1:covs[1]),convolutionBetaCumulative(x=c(0:covs[1]),cover=covs[1],beta=segInfo$beta,prob=0.5,nref=segInfo$nref))
      realValues <- computing$tn$ref
      segInfo$pval <- dgof::ks.test(x=realValues,y=computedCumulative)$p.value
    }    
    return(segInfo)
  }
  
  betaTable.list <- mclapply(segSeqAnalysis$results, getData, sampleName, mc.preschedule=T,mc.cores= n_corePerSample)  
  betaTable <- fromListToDF(betaTable.list)
  
  #sort betaTable
  #not: does not work with chr
  betaTable <- betaTable[order(as.numeric(sub("chr","",betaTable$chr)),betaTable$start),]
  
  return(betaTable)

}




#########################
#########################
# 
# Correct log2 data
#
#########################
#########################

# for(i in c(1:nrow(betaTable))){
#   cat(i,"\n")
#   percCorr <- as.numeric(correctionTable$Cov.Proportion[which(correctionTable$Sample %in% betaTable$sample[i])])
#   if (percCorr == "na"){
#     #return( NA )
#   }else{
#     aa<- as.numeric(x[columToCorrect]) - log2(percCorr)#return( as.numeric(x[columToCorrect]) - log2(percCorr) )      
#   }
# }
# 

#correct by ploidy
getLog2Corr <- function(betaTable, correctionTable,columToCorrect = 5){
  log2corr <- apply(betaTable,1,function(x){      
    #cat(x,"\n")
    percCorr <- as.numeric(correctionTable$Cov.Proportion[which(correctionTable$Sample %in% x[1])])
    if (is.na(percCorr)){
      return( NA )
    }else{
      return( as.numeric(x[columToCorrect]) - log2(percCorr) )      
    }  
  })
  return(log2corr)  
}


#correct by ploidy
ploidyShift <- function(betaTable,correctionTable){
  
  #thisTable <- by(betaTable,betaTable$sample,identity)[[1]]
  rownames(correctionTable) <- correctionTable$sample
  log2shifted.list <- by(betaTable,betaTable$sample,
                    function(thisTable){
                      sampleName <- thisTable$sample[1]
                      L2shift <- correctionTable[sampleName,"log2shift"]
                      return(thisTable$log2 - L2shift)
  })
  log2shifted <- do.call(c,log2shifted.list)

  return(log2shifted)  
}



#correct by adm.global
getLog2Corr.adm <- function(betaTable, adm.global, cns=seq(0.01,10,0.01)){
  
  
  #compute explectedLog2 data for all adm
  expectedLog2all <- t(sapply(adm.global$adm,function(x){    
    if(is.na(x)){
      l2 <- rep(NA,length(cns))
    }else{
      l2 <- getExpectedLog2Ratio(as.numeric(x),cns)
    }
    return(l2)
  }))
  colnames(expectedLog2all) <- cns
  
  #correct log2
  #x<-unlist(betaTable[1,])
  aa<-t(apply(betaTable,1,function(x){  
    admId <- which( adm.global$sample == x[1] )
    adm <- as.numeric(adm.global$adm[admId])    
    adm.min <- as.numeric(adm.global$adm.min[admId])
    adm.max <- as.numeric(adm.global$adm.max[admId])
    if(is.na(adm) || is.na(x[14])){
      return(c(NA,NA,NA,NA))
    }
    log2 <- as.numeric(x[14])
    expectedLog2 <- expectedLog2all[admId,]
    
    if ( min(expectedLog2) > log2 ){
      l2corr <- NA
    }else{
      l2corr <- log2( as.numeric(names(expectedLog2)[rank(c(as.numeric(expectedLog2),log2 ),ties.method="max")[length(expectedLog2)+1]])/2)
    }
    return(c(adm,adm.min,adm.max,l2corr))    
  }))
   
  #hist(betaTable$log2,breaks=1000)
  #hist(2*2^aa,breaks=100000)
  
  betaTable$adm.global <- aa[,1]
  betaTable$adm.global.min <- aa[,2]
  betaTable$adm.global.max <- aa[,3]
  betaTable$log2.corr <- aa[,4]
  
  return(betaTable)
}

#correct by adm.global SD models
getLog2Corr.adm.SDmodel <- function(betaTable, adm.global, cns=seq(0.01,10,0.01), ncores){
  
  
  #compute explectedLog2 data for all adm
  expectedLog2all <- t(sapply(adm.global$mean,function(x){    
    if(is.na(x)){
      l2 <- rep(NA,length(cns))
    }else{
      l2 <- getExpectedLog2Ratio(as.numeric(x),cns)
    }
    return(l2)
  }))
  colnames(expectedLog2all) <- cns
  
  #
  betaTable$adm.global <- NA
  betaTable$adm.global.min <- NA
  betaTable$adm.global.max <- NA
  betaTable$log2.corr <- NA
  betaTable$adm.global.sd <- NA
  
  
  betaTable.list <- mclapply(c(1:nrow(betaTable)),  
      function(i){
        x <-  betaTable[i,]
        
        admId <- which( adm.global$sample == x$sample )
        adm <- as.numeric(adm.global$mean[admId])    
        adm.min <- as.numeric(adm.global$Conf.lower[admId])
        adm.max <- as.numeric(adm.global$Conf.upper[admId])
        adm.sd <- as.numeric(adm.global$SD[admId])                
        
        if(is.na(adm) || is.na(x$log2.plCorr)){
          return(x)
        }
        log2 <- x$log2.plCorr
        expectedLog2 <- expectedLog2all[admId,]
        
        if ( min(expectedLog2) > log2 ){
          l2corr <- NA
        }else{
          l2corr <- log2( as.numeric(names(expectedLog2)[rank(c(as.numeric(expectedLog2),log2 ),ties.method="max")[length(expectedLog2)+1]])/2)
        }
        
        x$adm.global <- adm
        x$adm.global.min <-  adm.min
        x$adm.global.max <- adm.max
        x$adm.global.sd <- adm.sd
        x$log2.corr <- l2corr
        
        #return(c(adm,adm.min,adm.max,l2corr,adm.sd))    
        return(x)
      },mc.cores=ncores)
  
  betaTable <- fromListToDF(betaTable.list)
  
  
  return(betaTable)
}



#old function! remove when code is stable
getExpectedLog2Ratio <- function(adm.global,cn_state){
  #cn_state <- seq(0,10,0.01)
  num_states <- length(cn_state)
  expectedLog2Ratio <- matrix(data=NA, ncol=num_states, nrow=1)
  colnames(expectedLog2Ratio) <- c(cn_state) # paste("CN_",cn_state,sep=""))
  
  expectedLog2Ratio[1,] <- log2(( adm.global  + ( (1 - adm.global) * cn_state/2  ) ))
  return(expectedLog2Ratio)
}


getAdm <- function(adm.global, minSupportingMutations){
  #select adm.global from cn1 and cn1-3
  tmpAdm <- t(apply(adm.global,1,function(x){
    #cat(x,"\n")
    if (!is.na(x[10]) && as.numeric(x[10]) >= minSupportingMutations){
      x[6:8]
    }else if (!is.na(x[16]) && as.numeric(x[16]) >= minSupportingMutations){
      x[12:14]
    }else{
      c(NA,NA,NA)
    }
  }))
  
  adm.global$adm <- as.numeric(tmpAdm[,1])
  adm.global$adm.min <- as.numeric(tmpAdm[,2])
  adm.global$adm.max <- as.numeric(tmpAdm[,3])
    
  return(adm.global)
  
}


############################################
############################################
#
# Create clonality table from beta table with
# log2 corrected data
#
############################################
############################################

getClonalityWithError<-function(integerCN,
                                beta,
                                adm.global.thisZone,
                                adm.global.thisZone.min,
                                adm.global.thisZone.max,
                                clonalityThreshold,
                                local.error,
                                roundDec=3){
  if(integerCN == 0){
    
    betaCorr <- beta
    zone.ctm.local <- beta / (2 - beta)
    G <- adm.global.thisZone
    L <- zone.ctm.local
    clonality <- setBetween0and1(1- ((G - ( L * G)) / ( L * ( 1 - G))))
    
    zone.ctm.local.min <- zone.ctm.local - local.error
    zone.ctm.local.max <- zone.ctm.local + local.error
    
    G.int <- interval(adm.global.thisZone.min,adm.global.thisZone.max)
    L.int <- interval(zone.ctm.local.min,zone.ctm.local.max  )         
        
    clonality.int <-  1 - ((G.int - ( L.int * G.int)) / ( L.int * ( 1 - G.int)))      
    
    clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
  }
  
  if (integerCN == 1){
    betaCorr <- beta
    zone.ctm.local <- beta / (2 - beta)
    clonality <- setBetween0and1((1 - zone.ctm.local) / (1 - adm.global.thisZone))    
    
    zone.ctm.local.min <- zone.ctm.local - local.error
    zone.ctm.local.max <- zone.ctm.local + local.error
    
    zone.ctm.local.int <- interval(zone.ctm.local.min,zone.ctm.local.max)
    adm.global.thisZone.int <- interval(adm.global.thisZone.min,adm.global.thisZone.max)
    
    clonality.int <- ( 1 - zone.ctm.local.int) / (1 - adm.global.thisZone.int)
    
    clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
    
  }
  if (integerCN == 2){
    betaCorr <- beta
    zone.ctm.local <- betaCorr
    clonality <- zone.ctm.local  
    zone.ctm.local.min <- zone.ctm.local - local.error
    zone.ctm.local.max <- zone.ctm.local + local.error    
    clonality.int <- interval(zone.ctm.local.min,zone.ctm.local.max)
    
    clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
    
  }
  if (integerCN == 3){
    
    betaCorr  <-    1 - (3 * (1 - beta) )
    if ( betaCorr > 0){  
      zone.ctm.local <- ( 3 * betaCorr ) / (( 3 - 2 ) * betaCorr + 2)
      clonality <- setBetween0and1((1 - zone.ctm.local) / (1 - adm.global.thisZone))
            
      zone.ctm.local.min <- zone.ctm.local - local.error
      zone.ctm.local.max <- zone.ctm.local + local.error
      
      zone.ctm.local.int <- interval(zone.ctm.local.min,zone.ctm.local.max)
      adm.global.thisZone.int <- interval(adm.global.thisZone.min,adm.global.thisZone.max)
      
      clonality.int <- ( 1 - zone.ctm.local.int) / (1 - adm.global.thisZone.int)
      clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
    }else{
      betaCorr <- NA
      zone.ctm.local <- NA
      clonality <- NA
      clonality.int <- interval(NULL,NULL)
      clonality.status <- "not.analysed"
    }
        
  } 
  if (integerCN == 4){
    betaCorr <- beta    
    zone.ctm.local <- ( 4 * betaCorr ) / ((4-2) * betaCorr +2 ) 
    clonality <- setBetween0and1(zone.ctm.local  )
    zone.ctm.local.min <- zone.ctm.local - local.error
    zone.ctm.local.max <- zone.ctm.local + local.error    
    clonality.int <- interval(zone.ctm.local.min,zone.ctm.local.max)    
    clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
  }
  
  if (integerCN == 5){
    betaCorr  <-    1 - (5 * (1 - beta) )
    
    if ( betaCorr > 0){  
      zone.ctm.local <- ( 5 * betaCorr ) / (( 5 - 2 ) * betaCorr + 2)
      clonality <- setBetween0and1((1 - zone.ctm.local) / (1 - adm.global.thisZone))
      
      zone.ctm.local.min <- zone.ctm.local - local.error
      zone.ctm.local.max <- zone.ctm.local + local.error
      
      zone.ctm.local.int <- interval(zone.ctm.local.min,zone.ctm.local.max)
      adm.global.thisZone.int <- interval(adm.global.thisZone.min,adm.global.thisZone.max)
      
      clonality.int <- ( 1 - zone.ctm.local.int) / (1 - adm.global.thisZone.int)
      clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
    }else{
      betaCorr <- NA
      zone.ctm.local <- NA
      clonality <- NA
      clonality.int <- interval(NULL,NULL)
      clonality.status <- "not.analysed"
    }
  }
  
  if (integerCN > 5){
    betaCorr <- NA
    zone.ctm.local <- NA
    clonality <- NA
    clonality.int <- interval(NULL,NULL)
    clonality.status <- "not.analysed"
  }
    
  if (interval_is_empty(clonality.int)){
    clonality.min <- NA
    clonality.max <- NA    
  }else{    
    clonality <- round(clonality,roundDec)
    clonality.min <- round(min(clonality.int),roundDec)
    clonality.max <- round(max(clonality.int),roundDec)
    if ( clonality.min >= clonalityThreshold  ){ clonality.status <- "clonal"}
    if ( clonality.max <= clonalityThreshold ){ clonality.status <- "subclonal"}    
    if ( clonality.min <= clonalityThreshold  &&  clonality.max >= clonalityThreshold && clonality < clonalityThreshold ) { clonality.status <- "uncertain.subclonal" }
    if ( clonality.min <= clonalityThreshold &&  clonality.max >= clonalityThreshold && clonality >= clonalityThreshold ){ clonality.status <- "uncertain.clonal"}
  }
  
  data<-data.frame(row.names=1,stringsAsFactors=F)
  data$betaCorr <- betaCorr
  data$zone.ctm.local <- zone.ctm.local
  data$clonality <- clonality
  data$clonality.min <- clonality.min
  data$clonality.max <- clonality.max
  data$clonality.status <- clonality.status
  return(data)
}


computeClonalityTable <- function(betaTable,errorTable,clonalityThreshold,betaThreshold=NULL,roundDec = 3){

  ## backward compatibility
  if (is.null(betaThreshold)){betaThreshold<-clonalityThreshold}
  
  betaTable$integerCN <- NA
  betaTable$clonality <- NA
  betaTable$clonality.min <- NA
  betaTable$clonality.max <- NA
  betaTable$clonality.status <- "not.analysed"
  
  getIndexClonality <- function(i){
    #cat(i,"\n")
    zone <- betaTable[i,]
    zone$adm.global <- round(zone$adm.global,roundDec)
    zone$adm.global.min <- round(zone$adm.global.min,roundDec)
    zone$adm.global.max <- round(zone$adm.global.max,roundDec)
    
    #for(colN in c(1:ncol(zone)))
    
    #can be analysed
    if (is.na(zone$beta) || is.na(zone$log2.corr)){
      return(zone)
    }
    
    #determine error from nsnps and cov
    nsnps <- zone$nsnp
    cov <- zone$cov
    idxError <- which(errorTable$n.info.snps <= nsnps & errorTable$mean.cov <= cov)
    if (length(idxError) == 0 ){idxError <- 1}    
    local.error <- errorTable$adm.estimation.error[max(idxError)]
    
    #cn given log2
    cn <- 2*2^zone$log2.corr
    
    #valid cn are floor(cn) and ceil(cn)  
    clonCNlow <- getClonalityWithError(integerCN=floor(cn),
                                       beta=zone$beta,
                                       adm.global.thisZone=zone$adm.global,
                                       adm.global.thisZone.min=zone$adm.global.min,
                                       adm.global.thisZone.max=zone$adm.global.max,
                                       clonalityThreshold,
                                       local.error,
                                       roundDec)
    
    clonCNhigh <- getClonalityWithError(integerCN=ceiling(cn),
                                        beta=zone$beta,
                                        adm.global.thisZone=zone$adm.global,
                                        adm.global.thisZone.min=zone$adm.global.min,
                                        adm.global.thisZone.max=zone$adm.global.max,
                                        clonalityThreshold,
                                        local.error,
                                        roundDec)
    
    if (is.null(clonCNlow) | is.null(clonCNhigh) | is.na(clonCNlow$clonality) | is.na(clonCNhigh$clonality) ){
      return(zone)
    }
     
    
    
    if (cn >= 0 & cn < 1){
      #if ( ! clonCNhigh$clonality.status %in% c("clonal","uncertain.clonal")){
      #if ( betaThreshold 
      #  zone$integerCN <- floor(cn)
      #  zone$clonality <- clonCNlow$clonality
      #  zone$clonality.min <- clonCNlow$clonality.min
      #  zone$clonality.max <- clonCNlow$clonality.max
      #  zone$clonality.status <- clonCNlow$clonality.status
      #}else{
      #  zone$integerCN <- ceil(cn)
      #  zone$clonality <- clonCNhigh$clonality
      #  zone$clonality.min <- clonCNhigh$clonality.min
      #  zone$clonality.max <- clonCNhigh$clonality.max
      #  zone$clonality.status <- clonCNhigh$clonality.status
      #}
      if ( zone$beta >= betaThreshold ){
        zone$integerCN <- 0
        zone$clonality <- clonCNlow$clonality
        zone$clonality.min <- clonCNlow$clonality.min
        zone$clonality.max <- clonCNlow$clonality.max
        zone$clonality.status <- "clonal"
      }else if (cn <= 0.8) {
        zone$integerCN <- floor(cn)
        zone$clonality <- clonCNlow$clonality
        zone$clonality.min <- clonCNlow$clonality.min
        zone$clonality.max <- clonCNlow$clonality.max
        zone$clonality.status <- clonCNlow$clonality.status
      }else{
        zone$integerCN <- ceiling(cn)
        zone$clonality <- clonCNhigh$clonality
        zone$clonality.min <- clonCNhigh$clonality.min
        zone$clonality.max <- clonCNhigh$clonality.max
        zone$clonality.status <- clonCNhigh$clonality.status
      }
    }
    if (cn >= 1 & cn < 2){
      #if ( cn < 1+clonalityThreshold && ! clonCNhigh$clonality.status %in% c("clonal")){
      if (zone$beta <= betaThreshold){
        zone$integerCN <- floor(cn)
        zone$clonality <- clonCNlow$clonality
        zone$clonality.min <- clonCNlow$clonality.min
        zone$clonality.max <- clonCNlow$clonality.max
        zone$clonality.status <- clonCNlow$clonality.status
      }else{
        zone$integerCN <- ceiling(cn)
        zone$clonality <- clonCNhigh$clonality
        zone$clonality.min <- clonCNhigh$clonality.min
        zone$clonality.max <- clonCNhigh$clonality.max
        zone$clonality.status <- "not.analysed"
      }      
    }  
    if (cn >= 2 & cn <= 3){
      #if ( cn > 2 + ( 1 - clonalityThreshold) && ! clonCNlow$clonality.status %in% c("clonal")  ){
      if (zone$beta <= betaThreshold){
        zone$integerCN <- ceiling(cn)
        zone$clonality <- clonCNhigh$clonality
        zone$clonality.min <- clonCNhigh$clonality.min
        zone$clonality.max <- clonCNhigh$clonality.max
        zone$clonality.status <- clonCNhigh$clonality.status
      }else{
        zone$integerCN <- floor(cn)
        zone$clonality <- clonCNlow$clonality
        zone$clonality.min <- clonCNlow$clonality.min
        zone$clonality.max <- clonCNlow$clonality.max
        zone$clonality.status <- "not.analysed"
      }      
    }
      
    
    #if(zone$nsnp < minSNPs || zone$cov < minCov){
    #  zone$clonality.status <- "not.analysed"
    #}
    
    return(zone)
  }
  
  #i<-346
  clonalityTable.list <-lapply(c(1:nrow(betaTable)),getIndexClonality)
  clonalityTable <- fromListToDF(clonalityTable.list)
  
  return(clonalityTable)
  #hist(clonalityTable$ )
}


############################################
############################################
#
# Create clonality table from beta: SD error model


getClonalityWithError.SDmodel<-function(integerCN,
                                        beta,
                                        adm.global.thisZone,
                                        adm.global.thisZone.min,
                                        adm.global.thisZone.max,
                                        adm.global.thisZone.SD,
                                        clonalityThreshold,
                                        local.error.mean,
                                        local.error.SD){
#   if(integerCN == 0){
#     
#     betaCorr <- beta
#     zone.ctm.local <- beta / (2 - beta)
#     G <- adm.global.thisZone
#     L <- zone.ctm.local
#     clonality <- setBetween0and1(1- ((G - ( L * G)) / ( L * ( 1 - G))))
#     
#     zone.ctm.local.min <- zone.ctm.local - local.error
#     zone.ctm.local.max <- zone.ctm.local + local.error
#     
#     G.int <- c(adm.global.thisZone.min,adm.global.thisZone.max)
#     L.int <- c(zone.ctm.local.min,zone.ctm.local.max  )         
#     
#     clonality.int <-  expression(1 - ((G.int - ( L.int * G.int)) / ( L.int * ( 1 - G.int)))) 
#     dataF <- cbind(G.int,L.int)
#     
#     res <- propagate(expr = clonality.int, data = dataF, type = "stat", do.sim = F, do.perm=TRUE, verbose = F) 
#     
#     clonality.int <-  interval(res$summary["Conf.lower","Perm"],res$summary["Conf.upper","Perm"])
#     
#     clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
#   }
#   
#   if (integerCN == 1){
#     betaCorr <- beta
#     zone.ctm.local <- beta / (2 - beta)
#     clonality <- setBetween0and1((1 - zone.ctm.local) / (1 - adm.global.thisZone))    
#     
#     zone.ctm.local.min <- zone.ctm.local - local.error
#     zone.ctm.local.max <- zone.ctm.local + local.error
#     
#     zone.ctm.local.int <- interval(zone.ctm.local.min,zone.ctm.local.max)
#     adm.global.thisZone.int <- interval(adm.global.thisZone.min,adm.global.thisZone.max)
#     
#     clonality.int <- ( 1 - zone.ctm.local.int) / (1 - adm.global.thisZone.int)
#     
#     clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
#     
#   }
  if (integerCN == 2){
    
    coeffVariation.local <-  local.error.SD / local.error.mean
    SD.local <- coeffVariation.local * beta
    
    beta * local.error.SD
    
    
    err <- c( local.error.mean , local.error.SD)     
    beta.int <- c(beta,0)
    EXPR <- expression(-err+beta.int) 
    
    dataF <- cbind(err,beta.int)
    colnames(dataF)
    res <- propagate(expr = EXPR, data = dataF, type = "stat", do.sim = TRUE, do.perm=TRUE, verbose = TRUE) 
    res$summary
    
    G.int <- c(adm.global.thisZone.min,adm.global.thisZone.max)
    L.int <- c(zone.ctm.local.min,zone.ctm.local.max  )         
    clonality.int <-  expression(1 - ((G.int - ( L.int * G.int)) / ( L.int * ( 1 - G.int)))) 
    dataF <- cbind(G.int,L.int)
    colnames(dataF)
    res <- propagate(expr = clonality.int, data = dataF, type = "stat", do.sim = TRUE, do.perm=TRUE, verbose = TRUE) 
    res$summary
    
    betaCorr <- beta    
    
    
    
    
    zone.ctm.local <- betaCorr
    clonality <- zone.ctm.local
    
    
    
    
    
    zone.ctm.local.min <- zone.ctm.local - local.error
    zone.ctm.local.max <- zone.ctm.local + local.error    
    clonality.int <- interval(zone.ctm.local.min,zone.ctm.local.max)
    
    clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
    
  }
#   if (integerCN == 3){
#     
#     betaCorr  <-    1 - (3 * (1 - beta) )
#     if ( betaCorr > 0){  
#       zone.ctm.local <- ( 3 * betaCorr ) / (( 3 - 2 ) * betaCorr + 2)
#       clonality <- setBetween0and1((1 - zone.ctm.local) / (1 - adm.global.thisZone))
#       
#       zone.ctm.local.min <- zone.ctm.local - local.error
#       zone.ctm.local.max <- zone.ctm.local + local.error
#       
#       zone.ctm.local.int <- interval(zone.ctm.local.min,zone.ctm.local.max)
#       adm.global.thisZone.int <- interval(adm.global.thisZone.min,adm.global.thisZone.max)
#       
#       clonality.int <- ( 1 - zone.ctm.local.int) / (1 - adm.global.thisZone.int)
#       clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
#     }else{
#       betaCorr <- NA
#       zone.ctm.local <- NA
#       clonality <- NA
#       clonality.int <- interval(NULL,NULL)
#       clonality.status <- "not.analysed"
#     }
#     
#   } 
#   if (integerCN == 4){
#     betaCorr <- beta    
#     zone.ctm.local <- ( 4 * betaCorr ) / ((4-2) * betaCorr +2 ) 
#     clonality <- setBetween0and1(zone.ctm.local  )
#     zone.ctm.local.min <- zone.ctm.local - local.error
#     zone.ctm.local.max <- zone.ctm.local + local.error    
#     clonality.int <- interval(zone.ctm.local.min,zone.ctm.local.max)    
#     clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
#   }
#   
#   if (integerCN == 5){
#     betaCorr  <-    1 - (5 * (1 - beta) )
#     
#     if ( betaCorr > 0){  
#       zone.ctm.local <- ( 5 * betaCorr ) / (( 5 - 2 ) * betaCorr + 2)
#       clonality <- setBetween0and1((1 - zone.ctm.local) / (1 - adm.global.thisZone))
#       
#       zone.ctm.local.min <- zone.ctm.local - local.error
#       zone.ctm.local.max <- zone.ctm.local + local.error
#       
#       zone.ctm.local.int <- interval(zone.ctm.local.min,zone.ctm.local.max)
#       adm.global.thisZone.int <- interval(adm.global.thisZone.min,adm.global.thisZone.max)
#       
#       clonality.int <- ( 1 - zone.ctm.local.int) / (1 - adm.global.thisZone.int)
#       clonality.int <- interval(setBetween0and1(min(clonality.int)),setBetween0and1(max(clonality.int)))
#     }else{
#       betaCorr <- NA
#       zone.ctm.local <- NA
#       clonality <- NA
#       clonality.int <- interval(NULL,NULL)
#       clonality.status <- "not.analysed"
#     }
#   }
#   
#   if (integerCN > 5){
#     betaCorr <- NA
#     zone.ctm.local <- NA
#     clonality <- NA
#     clonality.int <- interval(NULL,NULL)
#     clonality.status <- "not.analysed"
#   }
  
  if (interval_is_empty(clonality.int)){
    clonality.min <- NA
    clonality.max <- NA    
  }else{    
    clonality <- round(clonality,roundDec)
    clonality.min <- round(min(clonality.int),roundDec)
    clonality.max <- round(max(clonality.int),roundDec)
    if ( clonality.min >= clonalityThreshold  ){ clonality.status <- "clonal"}
    if ( clonality.max <= clonalityThreshold ){ clonality.status <- "subclonal"}    
    if ( clonality.min <= clonalityThreshold  &&  clonality.max >= clonalityThreshold && clonality < clonalityThreshold ) { clonality.status <- "uncertain.subclonal" }
    if ( clonality.min <= clonalityThreshold &&  clonality.max >= clonalityThreshold && clonality >= clonalityThreshold ){ clonality.status <- "uncertain.clonal"}
  }
  
  data<-data.frame(row.names=1,stringsAsFactors=F)
  data$betaCorr <- betaCorr
  data$zone.ctm.local <- zone.ctm.local
  data$clonality <- clonality
  data$clonality.min <- clonality.min
  data$clonality.max <- clonality.max
  data$clonality.status <- clonality.status
  return(data)
}



computeClonalityTable.SDmodel <- function(betaTable,errorTable,clonalityThreshold,betaThreshold=NULL,minSNPs,minCov,roundDec = 3,alpha=0.01){
  
  ## backward compatibility
  if (is.null(betaThreshold)){betaThreshold<-clonalityThreshold}
  
  betaTable$integerCN <- NA
  betaTable$clonality <- NA
  betaTable$clonality.min <- NA
  betaTable$clonality.max <- NA
  betaTable$clonality.status <- "not.analysed"
  
  getIndexClonality <- function(i,minSNPs,minCov){
    # cat(i,"\n")
    zone <- betaTable[i,]
    zone$adm.global <- zone$adm.global
    zone$adm.global.min <- zone$adm.global.min
    zone$adm.global.max <- zone$adm.global.max
    zone$adm.global.SD <- zone$adm.global.sd
    #for(colN in c(1:ncol(zone)))
    
    #can be analysed
    if (is.na(zone$beta) || is.na(zone$log2.corr)){
      return(zone)
    }
    
    #determine error from nsnps and cov
    nsnps <- zone$nsnp
    cov <- zone$cov
    idxError <- which(errorTable$n.info.snps <= nsnps & errorTable$mean.cov <= cov)
    if (length(idxError) == 0 ){idxError <- 1}    
    local.error.mean <- errorTable$mean.adm.estimation.error[max(idxError)]
    local.error.SD <- errorTable$SD.adm.estimation.error[max(idxError)]
    
    #cn given log2
    cn <- 2*2^zone$log2.corr
    
    #valid cn are floor(cn) and ceil(cn)  
    clonCNlow <- getClonalityWithError.SDmodel(integerCN=floor(cn),
                                       beta=zone$beta,
                                       adm.global.thisZone=zone$adm.global,
                                       adm.global.thisZone.min=zone$adm.global.min,
                                       adm.global.thisZone.max=zone$adm.global.max,
                                       adm.global.thisZone.SD=zone$adm.global.SD,
                                       clonalityThreshold,
                                       local.error.mean,
                                       local.error.SD)
    
    clonCNhigh <- getClonalityWithError(integerCN=ceiling(cn),
                                        beta=zone$beta,
                                        adm.global.thisZone=zone$adm.global,
                                        adm.global.thisZone.min=zone$adm.global.min,
                                        adm.global.thisZone.max=zone$adm.global.max,
                                        clonalityThreshold,
                                        local.error,
                                        roundDec)
    
    if (is.null(clonCNlow) | is.null(clonCNhigh) | is.na(clonCNlow$clonality) | is.na(clonCNhigh$clonality) ){
      return(zone)
    }
    
    
    
    if (cn >= 0 & cn < 1){
      #if ( ! clonCNhigh$clonality.status %in% c("clonal","uncertain.clonal")){
      #if ( betaThreshold 
      #  zone$integerCN <- floor(cn)
      #  zone$clonality <- clonCNlow$clonality
      #  zone$clonality.min <- clonCNlow$clonality.min
      #  zone$clonality.max <- clonCNlow$clonality.max
      #  zone$clonality.status <- clonCNlow$clonality.status
      #}else{
      #  zone$integerCN <- ceil(cn)
      #  zone$clonality <- clonCNhigh$clonality
      #  zone$clonality.min <- clonCNhigh$clonality.min
      #  zone$clonality.max <- clonCNhigh$clonality.max
      #  zone$clonality.status <- clonCNhigh$clonality.status
      #}
      if ( zone$beta >= betaThreshold ){
        zone$integerCN <- 0
        zone$clonality <- clonCNlow$clonality
        zone$clonality.min <- clonCNlow$clonality.min
        zone$clonality.max <- clonCNlow$clonality.max
        zone$clonality.status <- "clonal"
      }else if (cn <= 0.8) {
        zone$integerCN <- floor(cn)
        zone$clonality <- clonCNlow$clonality
        zone$clonality.min <- clonCNlow$clonality.min
        zone$clonality.max <- clonCNlow$clonality.max
        zone$clonality.status <- clonCNlow$clonality.status
      }else{
        zone$integerCN <- ceiling(cn)
        zone$clonality <- clonCNhigh$clonality
        zone$clonality.min <- clonCNhigh$clonality.min
        zone$clonality.max <- clonCNhigh$clonality.max
        zone$clonality.status <- clonCNhigh$clonality.status
      }
    }
    if (cn >= 1 & cn < 2){
      #if ( cn < 1+clonalityThreshold && ! clonCNhigh$clonality.status %in% c("clonal")){
      if (zone$beta <= betaThreshold){
        zone$integerCN <- floor(cn)
        zone$clonality <- clonCNlow$clonality
        zone$clonality.min <- clonCNlow$clonality.min
        zone$clonality.max <- clonCNlow$clonality.max
        zone$clonality.status <- clonCNlow$clonality.status
      }else{
        zone$integerCN <- ceiling(cn)
        zone$clonality <- clonCNhigh$clonality
        zone$clonality.min <- clonCNhigh$clonality.min
        zone$clonality.max <- clonCNhigh$clonality.max
        zone$clonality.status <- "not.analysed"
      }      
    }  
    if (cn >= 2 & cn <= 3){
      #if ( cn > 2 + ( 1 - clonalityThreshold) && ! clonCNlow$clonality.status %in% c("clonal")  ){
      if (zone$beta <= betaThreshold){
        zone$integerCN <- ceiling(cn)
        zone$clonality <- clonCNhigh$clonality
        zone$clonality.min <- clonCNhigh$clonality.min
        zone$clonality.max <- clonCNhigh$clonality.max
        zone$clonality.status <- clonCNhigh$clonality.status
      }else{
        zone$integerCN <- floor(cn)
        zone$clonality <- clonCNlow$clonality
        zone$clonality.min <- clonCNlow$clonality.min
        zone$clonality.max <- clonCNlow$clonality.max
        zone$clonality.status <- "not.analysed"
      }      
    }
    
    
    if(zone$nsnp < minSNPs || zone$cov < minCov){
      zone$clonality.status <- "not.analysed"
    }
    
    return(zone)
  }
  
  #i<-346
  clonalityTable.list <-lapply(c(1:nrow(betaTable)),getIndexClonality,minSNPs=minSNPs,minCov=minCov)
  clonalityTable <- fromListToDF(clonalityTable.list)
  
  return(clonalityTable)
  #hist(clonalityTable$ )
}



#########################
#########################
# 
# Build summary table
#
#########################
#########################


#big table has a row for each intersection between a gene and a segment
# Gene.id chr start end sample  adm.global segstart segend log2corr %ofGeneCoverage integer.CN clonality clonality.min clonality.max status
getGeneStatus <- function(geneLine, geneList, clonalityTable){
  bedLine <- geneList[geneLine,]
  segsId <- getSegmentsPos(bedLine[1],bedLine[2],bedLine[3],segments=clonalityTable[,2:4])
  if (length(segsId)==0){
    ## NEED TO REPORT THAT NO DATA FOR THE GENE IS AVAILABLE!!!!
    cat("Warning: gene ", unlist(bedLine[4])," does not intersect any segment\n")    
    return(NULL)
  }
  nsegs <-length(segsId)
  geneTable <- data.frame(row.names=c(1:nsegs))
  
  geneTable$Gene.id <- rep(bedLine[4],nsegs)
  geneTable$chr <- rep(bedLine[1],nsegs)
  geneTable$start  <- rep(bedLine[2],nsegs)
  geneTable$end  <- rep(bedLine[3],nsegs)
  
  geneTable$sample <- clonalityTable$sample[segsId]
  geneTable$adm.global <- clonalityTable$adm.global[segsId]
  
  geneTable$seg.start <- clonalityTable$start[segsId]
  geneTable$seg.end <- clonalityTable$end[segsId]
  geneTable$log2 <- clonalityTable$log2.corr[segsId]
  
  #compute perc of gene represented by each segment
  overlaps <- apply(geneTable,1,function(x){
    computeSegmentsOverlap(x$start,x$end,x$seg.start,x$seg.end)
  } )    
  geneTable$perc.overlap <- round(overlaps / (unlist(geneTable$end) - unlist(geneTable$start) + 1),3)
  
  geneTable$integer.cn <- clonalityTable$integerCN[segsId]
  geneTable$clonality <- clonalityTable$clonality[segsId]
  geneTable$clonality.min <- clonalityTable$clonality.min[segsId]
  geneTable$clonality.max <- clonalityTable$clonality.max[segsId]
  geneTable$clonality.status <- clonalityTable$clonality.status[segsId]
  
  geneTable$nSNPs <- clonalityTable$nsnp[segsId]
  geneTable$cov <- clonalityTable$cov[segsId]
  geneTable$AFmean <- clonalityTable$AFmean[segsId]
  geneTable$AFsd <- clonalityTable$AFsd[segsId]
  
  return(geneTable)
  
}


#PMline<-304
#PMtable <- PMreadCounts
#clonalityTable <- clonalityTable
#getPMstatus <- function(PMline, PMtable, clonalityTable, AFthreshold, p.val = 0.05, expectedAF = 0.5, observedAF = 0.5 ){
getPMstatus <- function(PMline, PMtable, clonalityTable, p.val = 0.05, observedAF = 0.5 ){
  
  #cat(PMline,"\n")
  outTable <- data.frame(row.names=1)
  
  #this PM
  PM <- PMtable[PMline,]  
  
  if (is.na(PM$ref.count) | is.na(PM$alt.count)){
    return(NULL)
  }
  
  if (PM$ref.count + PM$alt.count == 0){
    return(NULL)
  }
  
  #this sample 
  clonalityTable.this <- clonalityTable[which(clonalityTable$sample == PM$sample),]
  
  if (nrow(clonalityTable.this) == 0){
    return(NULL)
  }
  
  
  
  segments.this <- cbind(clonalityTable.this$chr,clonalityTable.this$start,clonalityTable.this$end)
  #find seg
  PMseg <- getSegmentsPos(chr=PM$chr,initialPos=PM$start,finalPos=PM$end,segments.this)
  
  if( length(PMseg) == 0 ){ 
    err<- paste0("Warning: PM ",toString(PM)," - line number ",PMline," does not intersect segments \n")
    cat(err)
    return(NULL)
  }
  
  
  if( length(PMseg) > 1 ){ 
    err<- paste0("Error: PM ",toString(PM)," - line number ",PMline," intersect more than one segment\n")
    cat(err)
    stop()
  }
  
  seg <- clonalityTable.this[PMseg,]
  
  outTable$Gene.id <- PM$Gene.id
  outTable$chr <- PM$chr
  outTable$start <- PM$start
  outTable$end <- PM$end 
  outTable$sample <- PM$sample
  
  outTable$adm.global <- seg$adm.global
  outTable$seg.start <- seg$start
  outTable$seg.end <- seg$end
  outTable$log2 <- seg$log2.corr
  outTable$perc.overlap <- 1
  outTable$integer.cn <- seg$integerCN
  
  if (is.na(outTable$integer.cn)){
    outTable$clonality <- NA
    outTable$clonality.min <- NA
    outTable$clonality.max <- NA
    outTable$clonality.status <- "not.analysed"
    outTable$AFthreshold <- NA
    outTable$ref <- NA
    outTable$alt <- NA
    return(outTable)
  }
  
  if (outTable$integer.cn == 2){
    #remove adm.global
    
    refCorr <- round(PM$ref.count * (1 - outTable$adm.global))    
    refCorr.min <- round(PM$ref.count * (1 - seg$adm.global.max ))
    refCorr.max <- round(PM$ref.count * (1 - seg$adm.global.min ))
    cov <- refCorr + PM$alt.count
    cov.min <- refCorr.min + PM$alt.count
    cov.max <- refCorr.max + PM$alt.count
    
    outTable$clonality <- round(PM$alt.count / cov,3)
    outTable$clonality.min <- round(PM$alt.count / cov.max,3)
    outTable$clonality.max <- round(PM$alt.count / cov.min,3)
    
    if(outTable$clonality > observedAF){
      AFthreshold <- (qbinom(1-(p.val/2),cov,observedAF)+1)/cov
      if (outTable$clonality.min > AFthreshold && outTable$clonality.max > AFthreshold){outTable$clonality.status <- "subclonal"  }
      if (outTable$clonality.min < AFthreshold && outTable$clonality.max < AFthreshold){outTable$clonality.status <- "clonal"  }            
      if (outTable$clonality.min <= AFthreshold && outTable$clonality.max >= AFthreshold && outTable$clonality <= AFthreshold){
        outTable$clonality.status <- "uncertain.clonal"  
      }
      if (outTable$clonality.min <= AFthreshold && outTable$clonality.max >= AFthreshold && outTable$clonality > AFthreshold){
        outTable$clonality.status <- "uncertain.subclonal"
      }
      
    }else{
      AFthreshold <- max(0,(qbinom(p.val/2,cov,observedAF)-1)/cov)
      if (outTable$clonality.min < AFthreshold && outTable$clonality.max < AFthreshold){outTable$clonality.status <- "subclonal"  }
      if (outTable$clonality.min > AFthreshold && outTable$clonality.max > AFthreshold){outTable$clonality.status <- "clonal"  }
      if (outTable$clonality.min <= AFthreshold && outTable$clonality.max >= AFthreshold && outTable$clonality >= AFthreshold ){
        outTable$clonality.status <- "uncertain.clonal"  
      }
      if (outTable$clonality.min <= AFthreshold && outTable$clonality.max >= AFthreshold && outTable$clonality < AFthreshold ){
        outTable$clonality.status <- "uncertain.subclonal"
      }      
    }
    
  
    outTable$AFthreshold <- AFthreshold
    outTable$ref <- PM$ref.count
    outTable$alt <- PM$alt.count
    
  }else{
    refCorr <- round(PM$ref.count * (1 - outTable$adm.global))
    cov <- refCorr + PM$alt.count
    
    test <- binom.test(PM$alt.count,cov,0.5 )
    outTable$clonality <- test$estimate
    outTable$clonality.min <- test$conf.int[1]
    outTable$clonality.max <- test$conf.int[2]
    outTable$clonality.status <- "not.analysed"       
    outTable$AFthreshold <- NA
    outTable$ref <-refCorr
    outTable$alt <- PM$alt.count
  }
  return(outTable)    
}

#bpLine <- 2
#bpData <- countData  
#clonalityTable <- clonalityTable
getBPstatus <- function(bpLine, bpData, clonalityTable, geneList, p.val=0.01){
  
  #cat(bpLine,"\n")
  
  #this BP
  bp <- bpData[bpLine,]  
  outTableSeg1 <- data.frame(row.names=1)
  outTableSeg2 <- data.frame(row.names=1)
  
  minCov <- min(c(bp$breakPointer.cov,bp$BP1.cov,bp$BP2.cov,bp$splitted.cov,bp$BP1.cross.cov,bp$BP2.cross.cov ),na.rm=T)
  
  ##init tables for seg1 and seg2
  ##seg1
  outTableSeg1$Gene.id <- paste0("gene.on.chr",gsub("chr","",bp$chr1))
  outTableSeg1$chr <- bp$chr1
  outTableSeg1$start <- bp$pos1
  outTableSeg1$end <- bp$pos1
  outTableSeg1$sample <- bp$sample
  outTableSeg1$adm.global <- NA
  outTableSeg1$seg.start <- NA
  outTableSeg1$seg.end <- NA
  outTableSeg1$log2 <- NA
  outTableSeg1$perc.overlap <- NA
  outTableSeg1$integer.cn <- NA
  outTableSeg1$clonality <- NA
  outTableSeg1$clonality.min <- NA
  outTableSeg1$clonality.max <- NA
  outTableSeg1$clonality.status <- "not.analysed"
  outTableSeg1$gene.dist <- NA
  outTableSeg1$bp2.chr <- bp$chr2
  outTableSeg1$bp2.pos <- bp$pos2    
  outTableSeg1$APthr <- NA
  outTableSeg1$minCov <- minCov
  outTableSeg1$p.val <- bp$p.value
  outTableSeg1$class <- bp$dRanger.class
  
  #seg2
  outTableSeg2$Gene.id <- paste0("gene.on.chr",gsub("chr","",bp$chr2))
  outTableSeg2$chr <- bp$chr2
  outTableSeg2$start <- bp$pos2
  outTableSeg2$end <- bp$pos2
  outTableSeg2$sample <- bp$sample
  outTableSeg2$adm.global <- NA
  outTableSeg2$seg.start <- NA
  outTableSeg2$seg.end <- NA
  outTableSeg2$log2 <- NA
  outTableSeg2$perc.overlap <- NA
  outTableSeg2$integer.cn <- NA
  outTableSeg2$clonality <- NA
  outTableSeg2$clonality.min <- NA
  outTableSeg2$clonality.max <- NA
  outTableSeg2$clonality.status <- "not.analysed"
  outTableSeg2$gene.dist <- NA
  outTableSeg2$bp2.chr <- bp$chr1
  outTableSeg2$bp2.pos <- bp$pos1
  outTableSeg2$APthr <- NA
  outTableSeg2$minCov <- minCov
  outTableSeg2$p.val <- bp$p.value
  outTableSeg2$class <- bp$dRanger.class
  
  
  if ((! as.numeric(bp$chr1) %in% seq(1,22,1)) || (! as.numeric(bp$chr2) %in% seq(1,22,1))){    
    return(rbind(outTableSeg1,outTableSeg2))
  }
  
  ## find genes near bp
  findNearestGene <- function(chr,pos,geneList ){
    
    outTable <- data.frame(row.names=1)
    
    #subSet to this chr
    geneList.chr <- geneList[which(geneList[,1] == chr),]
    
    geneList.chr  <- geneList.chr[order(geneList.chr[,2],geneList.chr[,3]),]
    
    intersect <- geneList.chr[which(geneList.chr[,2] <= pos & geneList.chr[,3] >= pos),]
    
    if (nrow(intersect) > 0){
      outTable$Gene.id <-  intersect[1,4]
      outTable$chr <-  intersect[1,1]
      outTable$start <-  intersect[1,2]
      outTable$end <-  intersect[1,3]
      outTable$dist <- 1
    }else{
      endBefore <- tail(geneList.chr[which(geneList.chr[,3] <= pos),],n=1)
      startAfter <- head(geneList.chr[which(geneList.chr[,2] >= pos),],n=1)
      #the pos is near a gene
      
      if (nrow(endBefore) == 0){
        outTable$Gene.id <- startAfter[,4]
        outTable$chr <- startAfter[,1]
        outTable$start <- startAfter[,2]
        outTable$end <- startAfter[,3]
        outTable$dist <- startAfter[,2] - pos
      }else if (nrow(startAfter) == 0){
        outTable$Gene.id <- endBefore[,4]
        outTable$chr <- endBefore[,1]
        outTable$start <- endBefore[,2]
        outTable$end <- endBefore[,3]
        outTable$dist <- pos - endBefore[,3]
      }else if (pos - endBefore[,3] <= startAfter[,2] - pos){
        outTable$Gene.id <- endBefore[,4]
        outTable$chr <- endBefore[,1]
        outTable$start <- endBefore[,2]
        outTable$end <- endBefore[,3]
        outTable$dist <- pos - endBefore[,3]
      }else{
        outTable$Gene.id <- startAfter[,4]
        outTable$chr <- startAfter[,1]
        outTable$start <- startAfter[,2]
        outTable$end <- startAfter[,3]
        outTable$dist <- startAfter[,2] - pos
      } 
    }
    
    return(outTable)
  }
  
  
  bp1.gene <- findNearestGene(chr=bp$chr1,pos=bp$pos1,geneList=geneList)
  bp2.gene <- findNearestGene(chr=bp$chr2,pos=bp$pos2,geneList=geneList)  
  outTableSeg1$Gene.id <- bp1.gene$Gene.id
  outTableSeg1$gene.dist <- bp1.gene$dist  
  outTableSeg2$Gene.id <- bp2.gene$Gene.id
  outTableSeg2$gene.dist <- bp2.gene$dist
  
  
  ## load segments intersecting bps
  clonalityTable.this <- clonalityTable[which(clonalityTable$sample == bp$sample),]  
  if ( nrow(clonalityTable.this) == 0 ){
    return(rbind(outTableSeg1,outTableSeg2))
  }  
  segments.this <- cbind(clonalityTable.this$chr,clonalityTable.this$start,clonalityTable.this$end)
  #non overlapping segs
  segments.this[,3] <- segments.this[,3] - 1
  bp1seg <- getSegmentsPos(chr=bp$chr1,initialPos=bp$pos1,finalPos=bp$pos1,segments.this)
  bp2seg <- getSegmentsPos(chr=bp$chr2,initialPos=bp$pos2,finalPos=bp$pos2,segments.this)
  if( length(bp1seg) != 1 || length(bp2seg) != 1   ){ 
    err<- paste0("Error in BP ",toString(bp)," - line number ",bpLine,"\n One breakpoint is not in a known region\n")
    cat(err)
    return(rbind(outTableSeg1,outTableSeg2))
    #  stop(err)
  }    
  seg1 <- clonalityTable.this[bp1seg,]
  seg2 <- clonalityTable.this[bp2seg,]
  
  ## add data from segments
  
  ## seg1
  outTableSeg1$adm.global <- seg1$adm.global
  outTableSeg1$seg.start <- seg1$start
  outTableSeg1$seg.end <- seg1$end
  outTableSeg1$log2 <- seg1$log2.corr
  outTableSeg1$perc.overlap <- 1
  outTableSeg1$integer.cn <- seg1$integerCN
  ## seg2
  outTableSeg2$adm.global <- seg2$adm.global
  outTableSeg2$seg.start <- seg2$start
  outTableSeg2$seg.end <- seg2$end
  outTableSeg2$log2 <- seg2$log2.corr
  outTableSeg2$perc.overlap <- 1
  outTableSeg2$integer.cn <- seg2$integerCN
  
  getBeta <- function(adm,cn){
    return( (2 * adm) / ( 2 * adm + cn - cn * adm ) )
  }
  
  ## if integer CN is defined compute AP statistics
  if (!is.na(outTableSeg1$integer.cn) && !is.na(outTableSeg2$integer.cn)){    
      ## compute AP and AP corrected
      bpCov <- min(bp$BP1.cov,bp$BP2.cov)
      bpDen <-  bp$breakPointer.cov + bpCov 
            
      if ( outTableSeg1$integer.cn == 2 && outTableSeg2$integer.cn == 2 ){
        bpDenCorr <- bp$breakPointer.cov + max(bpCov - round(bpDen * ( seg1$adm.global  )),0)
        bpDenCorrMin <- bp$breakPointer.cov + max(bpCov - round(bpDen * ( seg1$adm.global.max  )),0)
        bpDenCorrMax <- bp$breakPointer.cov + max(bpCov - round(bpDen * ( seg1$adm.global.min  )),0)
      
        bpAP <- bp$breakPointer.cov / bpDenCorr        
        bpMinAP <- bp$breakPointer.cov / bpDenCorrMax
        bpMaxAP <- bp$breakPointer.cov / bpDenCorrMin
        
        outTableSeg1$clonality <- bpAP
        outTableSeg1$clonality.min <- bpMinAP
        outTableSeg1$clonality.max <- bpMaxAP
        
        outTableSeg2$clonality <- bpAP
        outTableSeg2$clonality.min <- bpMinAP
        outTableSeg2$clonality.max <- bpMaxAP
        
        if ( is.nan(bpAP)) {
          return(rbind(outTableSeg1,outTableSeg2))
        }
        
        #compute thresholds 
        if ( bpAP <= 0.5){
          APthreshold <- (qbinom((p.val/2),bpDenCorr,0.5))/bpDenCorr        
        }else{
          APthreshold <- (qbinom(1-(p.val/2),bpDenCorr,0.5))/bpDenCorr        
        }
        
        outTableSeg1$APthr <- APthreshold
        outTableSeg2$APthr <- APthreshold
        
        if ( outTableSeg1$APthr <= 0.5 ){
          if (outTableSeg1$clonality.min >= APthreshold && outTableSeg1$clonality.max >= APthreshold){ outTableSeg1$clonality.status <- "clonal" }
          if (outTableSeg1$clonality.min < APthreshold && outTableSeg1$clonality.max < APthreshold){ outTableSeg1$clonality.status <- "subclonal" }
          if (outTableSeg1$clonality.min < APthreshold && outTableSeg1$clonality.max >= APthreshold && outTableSeg1$clonality >= APthreshold){ outTableSeg1$clonality.status <- "uncertain.clonal" }
          if (outTableSeg1$clonality.min < APthreshold && outTableSeg1$clonality.max >= APthreshold && outTableSeg1$clonality < APthreshold){ outTableSeg1$clonality.status <- "uncertain.subclonal" }          
        }else{
          if (outTableSeg1$clonality.min <= APthreshold && outTableSeg1$clonality.max <= APthreshold){ outTableSeg1$clonality.status <- "clonal" }
          if (outTableSeg1$clonality.min > APthreshold && outTableSeg1$clonality.max > APthreshold){ outTableSeg1$clonality.status <- "subclonal" }
          if (outTableSeg1$clonality.min <= APthreshold && outTableSeg1$clonality.max > APthreshold && outTableSeg1$clonality <= APthreshold){ outTableSeg1$clonality.status <- "uncertain.clonal" }
          if (outTableSeg1$clonality.min <= APthreshold && outTableSeg1$clonality.max > APthreshold && outTableSeg1$clonality > APthreshold){ outTableSeg1$clonality.status <- "uncertain.subclonal" }        
        }
        
        outTableSeg2$clonality.status <- outTableSeg1$clonality.status
        
      }
  }      

  return(rbind(outTableSeg1,outTableSeg2))
  
}


#########################
#########################
# Report functions
#########################
#########################


#CLONETcomputing <- dataOut$results[[id]]$computing
plotAFclonet <- function(CLONETcomputing,refCol=rgb(254,105,21,maxColorValue=255),altCol=rgb(102,16,87,maxColorValue=255)){
  toPlot <- rbind(CLONETcomputing$tn$ref,CLONETcomputing$tn$alt)  
  cov <- toPlot[1,] + toPlot[2,]  
  barplot(toPlot/cov[1],col=c(refCol,altCol),border=c(refCol,altCol),names.arg=rep("",ncol(toPlot)),ylim=c(0,1))
}

plotAFpileup <- function(samplePileup,refCol=rgb(254,105,21,maxColorValue=255),altCol=rgb(102,16,87,maxColorValue=255)){
  nRefAlt <-  getRefAndAltPercAbs(samplePileup)
  cov <- computeCoverage(samplePileup)
  toPlot <- rbind(nRefAlt$ref/cov,nRefAlt$alt/cov)
  barplot(toPlot,col=c(refCol,altCol),border=c(refCol,altCol),names.arg=rep("",ncol(toPlot)),ylim=c(0,1))
}




#########################
#########################
# Ploidy functions
#########################
#########################



### Functions ------
findOptimumPloidy <- function(observedLog2Beta,maxCN,minAdm=0.1,maxAdm=0.7,stepCN=0.01,ncores){  
  
  stepAdm <- 0.1
  #actCN <- 3
  #actAdm <- 0.2
  confs <- expand.grid(adm=seq(minAdm,maxAdm,stepAdm),cn=seq(1,maxCN,stepCN), stringsAsFactors=F)
  #i<-1
  computeObservedRMSE.ind <- function(i, confs,observedLog2Beta,maxCN){
    Conf <-  confs[i,] #c(confs$adm[i],confs$cn[i])
    Conf$RMSE <- computeObservedRMSE(Conf=c(confs$adm[i],confs$cn[i]), observedLog2Beta,maxCN=3)
    return(Conf    )
  }
  
  confs.list <-mclapply(seq(1,nrow(confs),1),FUN=computeObservedRMSE.ind,confs=confs,observedLog2Beta=observedLog2Beta,maxCN=maxCN,mc.preschedule=T,mc.cores=ncores)
  confs <- fromListToDF(confs.list)
  
  AA<-confs[which(confs$RMSE <= quantile(confs$RMSE,probs=0.01)),]
  valToRet <- AA$cn[which.min(AA$RMSE)]
  return(valToRet)
  
}

computeObservedRMSE <- function(Conf,observedLog2Beta,maxCN ){
  nPoints <- maxCN + 1
  adm.local <- Conf[1]
  ploidy <- Conf[2]
  #at("adm=",adm.local," ploid=",ploidy,"\n")
  randomData <- data.frame(row.names=seq(1,nPoints,1))
  randomData$amd.local <- rep(adm.local,nPoints)
  randomData$ploidy <- rep(ploidy,nPoints)
  randomData$cnA <- c(0,seq(1,maxCN,1))
  randomData$cnB  <- c(0,seq(1,maxCN,1))
  syntheticSegs.list <- lapply(seq(1,nrow(randomData),1),perturbDataDF,randomData)
  syntheticSegs<-cbind(randomData,do.call(rbind,syntheticSegs.list))
  #plot(syntheticSegs$log2Val,syntheticSegs$betaVal.apparent,pch=20,ylim=c(0,1))
  #points(syntheticSegs$log2Val,syntheticSegs$betaVal.apparent,pch=20,col="orange2")
  expectedLog2Beta <- syntheticSegs[,c("log2Val","betaVal.apparent")]  
  colnames(expectedLog2Beta ) <- c("log2Val","betaVal")
  return(computeRMSE(observedLog2Beta=observedLog2Beta,expectedLog2Beta=expectedLog2Beta))
}

perturbData <- function(cnA,cnB,adm.local,MeanPloidy,coeffVariation=0.2){
  plT <- cnA+cnB 
  #pltSd <- plT * coeffVariation
  #plT <- max(0,plT + rnorm(n=1,sd=pltSd))
  plN<-2
  
  betaVal <- ( adm.local * plN )  / ( plT - adm.local * ( plT - plN ) )
  
  n.cells.mono.apparent <-  max(cnA,cnB) - min(cnA,cnB) #(cnA + cnB) - min(cnA,cnB)
  n.cells.bi.apparent <- 2*min(cnA,cnB)
  
  adm.local.apparent <- ( adm.local + (1-adm.local)*(n.cells.bi.apparent)) / ( adm.local + (1-adm.local)*(n.cells.bi.apparent) + (1-adm.local)*(n.cells.mono.apparent)  )
  betaVal.apparent <- ( adm.local.apparent * 2 )  / ( 1 + adm.local.apparent ) 
  
  #adm.local.apparent <- adm.local + (1-adm.local)*(min(cnA,cnB)/max(cnA,cnB))  
  #betaVal.apparent <- round(( adm.local.apparent * 2 )  / ( 1 + adm.local.apparent ) ,3 )
  #betaVal.apparent <- if (cnA==cnB){ 1 }else{ betaVal + (1-betaVal) * ( 2*min(cnA,cnB)/(cnA+cnB)) }  
  #adm.local.apparent <- betaVal.apparent / (2-betaVal.apparent)
  
  log2Val <- log2(( ( betaVal * plN + ( 1 - betaVal ) * plT ) / plN) / (MeanPloidy / plN))
  
  
  log2Val.corr <- log2(plT / plN)
  
  ## add noise to betaBav
  #betaValSD 
  
  #betaVal.apparent <- if (cnA==cnB){ 1 }else{ betaVal + ( 2*min(cnA,cnB)/(cnA+cnB))}  
  #((MeanPloidy/2)))  
  
  outDF <- data.frame(row.names=1)
  outDF$cnA <- cnA
  outDF$cnB <- cnB
  outDF$ploidy <- MeanPloidy
  outDF$adm.local <- adm.local
  outDF$adm.local.apparent <- adm.local.apparent
  outDF$betaVal <- betaVal
  outDF$betaVal.apparent <- betaVal.apparent
  outDF$log2Val <- log2Val
  outDF$log2Val.corr <- log2Val.corr
  #outDF$log2Val.Segmentation <- log2(plT/MeanPloidy)
  return(outDF)
}

perturbDataDF <- function(i,randomData){
  return(perturbData(cnA=randomData$cnA[i],cnB=randomData$cnB[i],adm.local=randomData$amd.local[i],MeanPloidy=randomData$ploidy[i]))
}

computeRMSE <- function(observedLog2Beta,expectedLog2Beta,nCores=1){
  #for each observed compute the minimum distance from all expected
  distances <- unlist(mclapply(seq(1,nrow(observedLog2Beta),1),
                               computeMinDistanceFromExpected,
                               observedLog2Beta=observedLog2Beta,
                               expectedLog2Beta=expectedLog2Beta,mc.preschedule=T,mc.cores=nCores))
  RMSE <-  sqrt(sum(distances^2)/length(distances))
  return(RMSE)
}

computeMinDistanceFromExpected <- function(i,observedLog2Beta,expectedLog2Beta){
  thisLog2 <- observedLog2Beta$log2[i]
  return(min(abs(expectedLog2Beta$log2Val - thisLog2),na.rm=T))  
}

#for testing
getPossibleCnComb <- function(maxCN,step=1){
  out<-c()
  for(i in seq(0,maxCN,step)){
    for(j in seq(0,i,step)){
      out <- rbind(out,c(j,(i-j)))
    }
  }
  return(out)
}


computeSamplePloidy <- function(currentSample, betaTable.good, maxHomoDels,maxCN,nCoresPerSample ){
  
  cat("Processing sample ",currentSample,"\n")
  betaTable.sample <- betaTable.good[which(betaTable.good$sample == currentSample),]
  
  ## remove potential homo dels
  betaTable.sample.good <- betaTable.sample[which(betaTable.sample$log2 > quantile(betaTable.sample$log2,probs=maxHomoDels)),]
  
  if (nrow(betaTable.sample.good) > 0 ){
    ## prepare data
    observedLog2Beta <- data.frame(row.names=seq(1,nrow(betaTable.sample.good),1))
    observedLog2Beta$log2 <-  betaTable.sample.good$log2
    observedLog2Beta$beta <-  betaTable.sample.good$beta
    maxCN <- ceiling( 2/(2^min(observedLog2Beta$log2)) * 10)/10
    plComputed <- findOptimumPloidy(observedLog2Beta,maxCN=maxCN,ncores=nCoresPerSample )
    
  }else{
    plComputed <- NA  
  }
  dfOut <- data.frame(row.names=1,stringsAsFactors=F)
  dfOut$sample <- currentSample
  dfOut$ploidy <- plComputed    
  
  return(dfOut)  
  ## tests
#   plot(betaTable.sample$log2,betaTable.sample$beta,pch=20,col="gray70",xlim=c(-3,log2(maxCN/2)),ylim=c(0,1))
#   #plot(betaTable.sample.good$log2,betaTable.sample.good$beta,pch=20,col="yellow",xlim=c(-3,log2(maxCN/2)),ylim=c(0,1))
#   points(observedLog2Beta$log2,observedLog2Beta$beta,pch=20,col="red4")    
#   adm.local.test <- 0.6
#   set.seed(234)
#   nArms<-1000
#   SDcn <- 0.05
#   randomData <- data.frame(row.names=seq(1,nArms,1))
#   randomData$amd.local <- rep(adm.local.test,nArms)
#   randomData$ploidy <- rep(plComputed,nArms)
#   possibleCNs <- getPossibleCnComb(maxCN)
#   randomCNslines <- sample(seq(1,nrow(possibleCNs),1),size=nArms,replace=T)
#   randomData$cnA <- possibleCNs[randomCNslines,1]
#   randomData$cnB <- possibleCNs[randomCNslines,2]
#   noiseCNA <- rnorm(nArms,mean=0,sd=SDcn  )
#   noiseCNB <- rnorm(nArms,mean=0,sd=SDcn  )
#   randomData$cnA <- pmax(0,randomData$cnA + noiseCNA)
#   randomData$cnB <- pmax(0,randomData$cnB + noiseCNB)
#   syntheticSegs.list <- lapply(seq(1,nrow(randomData),1),perturbDataDF,randomData)
#   syntheticSegs<-cbind(randomData,do.call(rbind,syntheticSegs.list))
#   #points(syntheticSegs$log2Val,syntheticSegs$betaVal.apparent,pch=20)
#   syntheticSegsGood <- syntheticSegs[which(syntheticSegs$betaVal.apparent >= 0.95),]
#   points(syntheticSegsGood$log2Val,syntheticSegsGood$betaVal.apparent,pch=20)  
  
} 












#########################
#########################
# garbage
#########################
#########################


# computeClonality.zone2<- function( zone.CN, beta, adm.global.thisZone,clonalityThreshold,minClonality){
#   #if( zone.CN > cn_levels[length(cn_levels)]){
#   #  cat("Warning! observed cn exceed cn_levels.\n")
#   #  return(NULL)
#   #}
#   #integerCN <- rank(c(cn_levels,zone.CN),ties.method="min")[length(cn_levels)+1]-1
#   #integerCN <- round(zone.CN)
#   clonalityInfo <- data.frame(row.names="1")
#   clonalityInfo$integerCN <- NA
#   clonalityInfo$zone.ctm.local <- NA
#   clonalityInfo$clonality <- NA
#   clonalityInfo$clonality.status <- NA
#   
#   
#   #if some data is missing
#   if (is.na(beta) | is.na(adm.global.thisZone) | is.na(zone.CN)){
#     return(clonalityInfo)
#   }
#   
#   #use clonalityThreshold to guess intege copy number and perform beta correction
#   #queryIntegerCN <- seq(0,8,1)
#   if ( ceil(zone.CN) - minClonality <= zone.CN ){
#     clonalityInfo$integerCN <- ceil(zone.CN)
#   }else{
#     clonalityInfo$integerCN <- floor(zone.CN)
#   }
#   
#   
#   
#   #CN=0 has to be treated separately
#   if (clonalityInfo$integerCN == 0 ){
#     zone.ctm.local <- beta / (2 - beta)
#     clonalityInfo$zone.ctm.local <- zone.ctm.local
#     G <- adm.global.thisZone
#     L <- zone.ctm.local
#     clonalityHomo <- (G - ( L * G)) / ( L * ( 1 - G))
#     clonalityInfo$clonality <- min(1,clonalityHomo)
#     
#     #check if integer CN is not corrrectly assigned
#     if ( clonalityInfo$clonality < minClonality ){
#       clonalityInfo$integerCN <- 1
#     }else{
#       if ( clonalityInfo$clonality >= clonalityThreshold  ){ clonalityInfo$clonality.status <- 0}
#       if ( clonalityInfo$clonality < clonalityThreshold ){ clonalityInfo$clonality.status <- 1}      
#       return(clonalityInfo)
#     }
#   }
#   
#   if (clonalityInfo$integerCN == 1){
#     zone.ctm.local <- beta / (2 - beta)
#     clonalityInfo$zone.ctm.local <- zone.ctm.local
#     zone.clonality <- (1 - zone.ctm.local) / (1 - adm.global.thisZone)
#     zone.clonality <- min(zone.clonality, 2 - zone.clonality)
#     clonalityInfo$clonality <- zone.clonality
#     if( clonalityInfo$clonality < minClonality ){
#       clonalityInfo$integerCN <- 2
#     }else{
#       if ( clonalityInfo$clonality >= clonalityThreshold  ){ clonalityInfo$clonality.status <- 0}
#       if ( clonalityInfo$clonality < clonalityThreshold ){ clonalityInfo$clonality.status <- 1}
#       return(clonalityInfo)
#     }
#   }
#   
#   if (clonalityInfo$integerCN == 2){
#     zone.ctm.local <- beta
#     clonalityInfo$zone.ctm.local <- zone.ctm.local
#     clonalityInfo$clonality <- zone.ctm.local
#     if( clonalityInfo$clonality < minClonality ){
#       clonalityInfo$integerCN <- 3
#     }else{
#       if ( clonalityInfo$clonality >= clonalityThreshold  ){ clonalityInfo$clonality.status <- 0}
#       if ( clonalityInfo$clonality < clonalityThreshold ){ clonalityInfo$clonality.status <- 1}
#       return(clonalityInfo)
#     }
#   }
#   
#   if (clonalityInfo$integerCN == 3){
#     beta  <-    1 - (3 * (1 - beta) / 2)
#     zone.ctm.local <- ( 3 * beta ) / (( 3 - 2 ) * beta + 2)
#     clonalityInfo$zone.ctm.local <- zone.ctm.local
#     zone.clonality <- (1 - zone.ctm.local) / (1 - adm.global.thisZone)
#     zone.clonality <- min(zone.clonality, 2 - zone.clonality)
#     clonalityInfo$clonality <- zone.clonality
#     if( clonalityInfo$clonality < minClonality ){
#       clonalityInfo$integerCN <- 4
#     }else{
#       if ( clonalityInfo$clonality >= clonalityThreshold  ){ clonalityInfo$clonality.status <- 0}
#       if ( clonalityInfo$clonality < clonalityThreshold ){ clonalityInfo$clonality.status <- 1}
#       return(clonalityInfo)
#     }
#   }
#   
#   if (clonalityInfo$integerCN > 3){
#     clonalityInfo$integerCN <- NA
#     clonalityInfo$zone.ctm.local <- NA
#     clonalityInfo$clonality <- NA
#     clonalityInfo$clonality.status <- NA
#     return(clonalityInfo)
#   }
#   
#   #return(c(zone.clonality,zone.clonality.status,integerCN))
# }
# 
# #compute clonality zoneCN <- regionData$zone.CNcorr, cn_levels,zone.ctm.local,regionData$adm.global.thisZone,cloanlityThreshold
# computeClonality.zone.v1 <- function( zone.CN, zone.ctm.local,adm.global.thisZone,clonalityThreshold){
#   #if( zone.CN > cn_levels[length(cn_levels)]){
#   #  cat("Warning! observed cn exceed cn_levels.\n")
#   #  return(NULL)
#   #}
#   #integerCN <- rank(c(cn_levels,zone.CN),ties.method="min")[length(cn_levels)+1]-1
#   #integerCN <- round(zone.CN)
#   
#   if (is.na(zone.ctm.local) | is.na(adm.global.thisZone) | is.na(zone.CN)){
#     return(c(NA,NA,NA))
#   }
#   
#   #queryIntegerCN <- seq(0,8,1)
#   if ( floor(zone.CN) + clanalityThreshold <= zone.CN ){
#     integerCN <- ceil(zone.CN)
#   }else{
#     integerCN <- floor(zone.CN)
#   }
#   
#   if (integerCN > 1){
#     zone.clonality <- NA
#     zone.clonality.status <- NA
#   }
#   if (integerCN == 1){
#     zone.clonality <- min(1,(1 - zone.ctm.local) / (1 - adm.global.thisZone))
#     if ( zone.clonality >= clonalityThreshold  ){ zone.clonality.status <- 0}
#     if ( zone.clonality < clonalityThreshold ){ zone.clonality.status <- 1}
#   }
#   if (integerCN == 0 ){
#     G <- adm.global.thisZone
#     L <- zone.ctm.local
#     clonalityHomo <- (G - ( L * G)) / ( L * ( 1 - G))
#     zone.clonality <- min(1,clonalityHomo)
#     if ( zone.clonality >= clonalityThreshold  ){ zone.clonality.status <- 0}
#     if ( zone.clonality < clonalityThreshold ){ zone.clonality.status <- 1}      
#   }        
#   return(c(zone.clonality,zone.clonality.status,integerCN))
# }



#start compute clonaliy of gene  here
# getClonality<-function(integerCN,beta,adm.global.thisZone,clonalityThreshold ){
#   if(integerCN == 0){
#     betaCorr <- beta
#     zone.ctm.local <- beta / (2 - beta)
#     G <- adm.global.thisZone
#     L <- zone.ctm.local
#     clonality <- (G - ( L * G)) / ( L * ( 1 - G))
#     clonality <- min(1,clonality)
#   }
#   if (integerCN == 1){
#     betaCorr <- beta
#     zone.ctm.local <- beta / (2 - beta)
#     clonality <- (1 - zone.ctm.local) / (1 - adm.global.thisZone)
#     clonality <- min(clonality, 2 - clonality)
#   }
#   if (integerCN == 2){
#     betaCorr <- beta
#     zone.ctm.local <- beta
#     clonality <- zone.ctm.local
#   }
#   if (integerCN == 3){
#     betaCorr  <-    1 - (3 * (1 - beta) / 2)
#     zone.ctm.local <- ( 3 * betaCorr ) / (( 3 - 2 ) * betaCorr + 2)
#     clonality <- (1 - zone.ctm.local) / (1 - adm.global.thisZone)
#     clonality <- min(clonality, 2 - clonality)
#   } 
#   if (integerCN > 3){
#     return(NULL)
#   }
#   if ( clonality >= clonalityThreshold  ){ clonality.status <- 0}
#   if ( clonality < clonalityThreshold ){ clonality.status <- 1}    
#   data<-data.frame(row.names=1,stringsAsFactors=F)
#   data$betaCorr <- betaCorr
#   data$zone.ctm.local <- zone.ctm.local
#   data$clonality <- clonality
#   data$clonality.status <- clonality.status
#   return(data)
# }


#compute clonality of a zone
# getIndexClonality <- function(i){
#   #cat(i,"\n")
#   zone <- betaTable[i,]
#   #can be analysed
#   if (is.na(zone$beta) | is.na(zone$log2.corr)){
#     return(zone)
#   }
#   
#   #determine error from nsnps and cov
#   nsnps <- zone$nsnp
#   cov <- zone$cov
#   idxError <- which(errorTable$n.info.snps <= nsnps & errorTable$mean.cov <= cov)
#   if (length(idxError) == 0 ){idxError <- 1}    
#   local.error <- errorTable$adm.estimation.error[max(idxError)]
#   
#   #cn given log2
#   cn <- 2*2^zone$log2.corr
#   
#   #valid cn are floor(cn) and ceil(cn)  
#   clonCNlow <- getClonalityWithError(integerCN=floor(cn),
#                                      beta=zone$beta,
#                                      adm.global.thisZone=zone$adm.global,
#                                      adm.global.thisZone.min=zone$adm.global.min,
#                                      adm.global.thisZone.max=zone$adm.global.max,
#                                      clonalityThreshold,
#                                      local.error)
#   
#   clonCNhigh <- getClonalityWithError(integerCN=ceil(cn),
#                                       beta=zone$beta,
#                                       adm.global.thisZone=zone$adm.global,
#                                       adm.global.thisZone.min=zone$adm.global.min,
#                                       adm.global.thisZone.max=zone$adm.global.max,
#                                       clonalityThreshold,
#                                       local.error)
#   
#   if (is.null(clonCNlow) | is.null(clonCNhigh)){
#     return(zone)
#   }
#   
#   
#   #cols are for low cn
#   #rows are for high cn
#   cnGT2mat <- rbind(c("L","H","H","H"),
#                     c("L","H","L","H"),
#                     c("L","H","L","H"),
#                     c("L","H","H","H")
#   )
#   rownames(cnGT2mat) <- c("clonal","subclonal","uncertain","not.analysed")
#   colnames(cnGT2mat) <- c("clonal","subclonal","uncertain","not.analysed")
#   
#   
#   cnLT2mat <- rbind(c("H","H","H","H"),
#                     c("L","L","L","L"),
#                     c("L","H","H","H"),
#                     c("L","L","L","L")
#   )
#   rownames(cnLT2mat) <- c("clonal","subclonal","uncertain","not.analysed")
#   colnames(cnLT2mat) <- c("clonal","subclonal","uncertain","not.analysed")
#   
#   if(clonCNhigh$clonality.status %in% c("uncertain.clonal","uncertain.subclonal")){
#     row <- "uncertain"
#   }else{
#     row <- clonCNhigh$clonality.status
#   }
#   
#   if(clonCNlow$clonality.status %in% c("uncertain.clonal","uncertain.subclonal")){
#     col <- "uncertain"
#   }else{
#     col <- clonCNlow$clonality.status
#   }
#   
#   if( cn < 2){
#     selected <- cnLT2mat[row,col]
#   }else{
#     selected <- cnGT2mat[row,col]              
#   }
#   
#   if (selected == "L"){
#     zone$integerCN <- floor(cn)
#     zone$clonality <- clonCNlow$clonality
#     zone$clonality.min <- clonCNlow$clonality.min
#     zone$clonality.max <- clonCNlow$clonality.max
#     zone$clonality.status <- clonCNlow$clonality.status
#     
#   }else{
#     zone$integerCN <- ceil(cn)
#     zone$clonality <- clonCNhigh$clonality
#     zone$clonality.min <- clonCNhigh$clonality.min
#     zone$clonality.max <- clonCNhigh$clonality.max
#     zone$clonality.status <- clonCNhigh$clonality.status      
#   }
#   
#   return(zone)
# }













#     #cols are for low cn
#     #rows are for high cn
#     cnGT2mat <- rbind(c("L","H","H","H"),
#                       c("L","H","L","H"),
#                       c("L","H","L","H"),
#                       c("L","H","H","H")
#     )
#     rownames(cnGT2mat) <- c("clonal","subclonal","uncertain","not.analysed")
#     colnames(cnGT2mat) <- c("clonal","subclonal","uncertain","not.analysed")
#     
#     
#     cnLT2mat <- rbind(c("H","H","H","H"),
#                       c("L","L","L","L"),
#                       c("L","H","H","H"),
#                       c("L","L","L","L")
#     )
#     rownames(cnLT2mat) <- c("clonal","subclonal","uncertain","not.analysed")
#     colnames(cnLT2mat) <- c("clonal","subclonal","uncertain","not.analysed")
#     
#     if(clonCNhigh$clonality.status %in% c("uncertain.clonal","uncertain.subclonal")){
#       row <- "uncertain"
#     }else{
#       row <- clonCNhigh$clonality.status
#     }
#     
#     if(clonCNlow$clonality.status %in% c("uncertain.clonal","uncertain.subclonal")){
#       col <- "uncertain"
#     }else{
#       col <- clonCNlow$clonality.status
#     }
#     
#     if( cn < 2){
#       selected <- cnLT2mat[row,col]
#     }else{
#       selected <- cnGT2mat[row,col]              
#     }
#     
#     if (cn <= 2){
#       if (clonCNlow$clonality > 1- clonalityThreshold){
#           zone$integerCN <- floor(cn)
#           zone$clonality <- clonCNlow$clonality
#           zone$clonality.min <- clonCNlow$clonality.min
#           zone$clonality.max <- clonCNlow$clonality.max
#           zone$clonality.status <- clonCNlow$clonality.status  
#       }else{
#           zone$integerCN <- ceil(cn)
#           zone$clonality <- clonCNhigh$clonality
#           zone$clonality.min <- clonCNhigh$clonality.min
#           zone$clonality.max <- clonCNhigh$clonality.max
#           zone$clonality.status <- clonCNhigh$clonality.status
#       }
#     }else{
#       if (clonCNhigh$clonality > 1- clonalityThreshold){
#         zone$integerCN <- ceil(cn)
#         zone$clonality <- clonCNhigh$clonality
#         zone$clonality.min <- clonCNhigh$clonality.min
#         zone$clonality.max <- clonCNhigh$clonality.max
#         zone$clonality.status <- clonCNhigh$clonality.status
#       }else{        
#         zone$integerCN <- floor(cn)
#         zone$clonality <- clonCNlow$clonality
#         zone$clonality.min <- clonCNlow$clonality.min
#         zone$clonality.max <- clonCNlow$clonality.max
#         zone$clonality.status <- clonCNlow$clonality.status  
#       }    
#     }




#     if( cn < 1){
#       if (clonCNlow$clonality > 1- clonalityThreshold){
#         zone$integerCN <- floor(cn)
#         zone$clonality <- clonCNlow$clonality
#         zone$clonality.min <- clonCNlow$clonality.min
#         zone$clonality.max <- clonCNlow$clonality.max
#         zone$clonality.status <- clonCNlow$clonality.status  
#       }else{
#         zone$integerCN <- ceil(cn)
#         zone$clonality <- clonCNhigh$clonality
#         zone$clonality.min <- clonCNhigh$clonality.min
#         zone$clonality.max <- clonCNhigh$clonality.max
#         zone$clonality.status <- clonCNhigh$clonality.status
#       }
#     }
#     if( cn >= 1 & cn < 2){
#       if (clonCNlow$clonality > 1- clonalityThreshold){
#         zone$integerCN <- floor(cn)
#         zone$clonality <- clonCNlow$clonality
#         zone$clonality.min <- clonCNlow$clonality.min
#         zone$clonality.max <- clonCNlow$clonality.max
#         zone$clonality.status <- clonCNlow$clonality.status  
#       }else{
#         zone$integerCN <- ceil(cn)
#         zone$clonality <- clonCNhigh$clonality
#         zone$clonality.min <- clonCNhigh$clonality.min
#         zone$clonality.max <- clonCNhigh$clonality.max
#         zone$clonality.status <- clonCNhigh$clonality.status
#       }
#     }
#     if( cn >= 2 & cn < 3){
#       if (clonCNlow$clonality > 1- clonalityThreshold){
#         zone$integerCN <- floor(cn)
#         zone$clonality <- clonCNlow$clonality
#         zone$clonality.min <- clonCNlow$clonality.min
#         zone$clonality.max <- clonCNlow$clonality.max
#         zone$clonality.status <- clonCNlow$clonality.status  
#       }else{
#         zone$integerCN <- ceil(cn)
#         zone$clonality <- clonCNhigh$clonality
#         zone$clonality.min <- clonCNhigh$clonality.min
#         zone$clonality.max <- clonCNhigh$clonality.max
#         zone$clonality.status <- clonCNhigh$clonality.status
#       }
#     }


#     else if (selected == "L"){
#       zone$integerCN <- floor(cn)
#       zone$clonality <- clonCNlow$clonality
#       zone$clonality.min <- clonCNlow$clonality.min
#       zone$clonality.max <- clonCNlow$clonality.max
#       zone$clonality.status <- clonCNlow$clonality.status
#       
#     }else{
#       zone$integerCN <- ceil(cn)
#       zone$clonality <- clonCNhigh$clonality
#       zone$clonality.min <- clonCNhigh$clonality.min
#       zone$clonality.max <- clonCNhigh$clonality.max
#       zone$clonality.status <- clonCNhigh$clonality.status      
#     }

#bpLine <- 2914
#bpData <- RR.AP  
#clonalityTable <- ClonalityTable
# getBPstatus <- function(bpLine, bpData, clonalityTable, geneList, p.val=0.001){
#   
#   #cat(bpLine,"\n")
#   
#   #this BP
#   bp <- bpData[bpLine,]  
#   outTableSeg1 <- data.frame(row.names=1)
#   outTableSeg2 <- data.frame(row.names=1)
#   
#   ##init tables for seg1 and seg2
#   ##seg1
#   outTableSeg1$Gene.id <- paste0("gene.on.chr",gsub("chr","",bp$chr1))
#   outTableSeg1$chr <- bp$chr1
#   outTableSeg1$start <- bp$pos1
#   outTableSeg1$end <- bp$pos1
#   outTableSeg1$sample <- bp$sample
#   outTableSeg1$adm.global <- NA
#   outTableSeg1$seg.start <- NA
#   outTableSeg1$seg.end <- NA
#   outTableSeg1$log2 <- NA
#   outTableSeg1$perc.overlap <- NA
#   outTableSeg1$integer.cn <- NA
#   outTableSeg1$clonality <- NA
#   outTableSeg1$clonality.min <- NA
#   outTableSeg1$clonality.max <- NA
#   outTableSeg1$clonality.status <- "not.analysed"
#   outTableSeg1$gene.dist <- NA
#   outTableSeg1$bp2.chr <- bp$chr2
#   outTableSeg1$bp2.pos <- bp$pos2    
#   outTableSeg1$APthr <- NA
#   outTableSeg1$minCov <- bp$minCov
#   outTableSeg1$p.val <- bp$p.value
#   outTableSeg1$class <- bp$dRanger.class
#   
#   #seg2
#   outTableSeg2$Gene.id <- paste0("gene.on.chr",gsub("chr","",bp$chr2))
#   outTableSeg2$chr <- bp$chr2
#   outTableSeg2$start <- bp$pos2
#   outTableSeg2$end <- bp$pos2
#   outTableSeg2$sample <- bp$sample
#   outTableSeg2$adm.global <- NA
#   outTableSeg2$seg.start <- NA
#   outTableSeg2$seg.end <- NA
#   outTableSeg2$log2 <- NA
#   outTableSeg2$perc.overlap <- NA
#   outTableSeg2$integer.cn <- NA
#   outTableSeg2$clonality <- NA
#   outTableSeg2$clonality.min <- NA
#   outTableSeg2$clonality.max <- NA
#   outTableSeg2$clonality.status <- "not.analysed"
#   outTableSeg2$gene.dist <- NA
#   outTableSeg2$bp2.chr <- bp$chr1
#   outTableSeg2$bp2.pos <- bp$pos1
#   outTableSeg2$APthr <- NA
#   outTableSeg2$minCov <- bp$minCov
#   outTableSeg2$p.val <- bp$p.value
#   outTableSeg2$class <- bp$dRanger.class
#   
#   
#   if (bp$chr1 > 22 || bp$chr2 > 22){    
#     return(rbind(outTableSeg1,outTableSeg2))
#   }
#   
#   ## find genes near bp
#   findNearestGene <- function(chr,pos,geneList ){
#     
#     outTable <- data.frame(row.names=1)
#     
#     #subSet to this chr
#     geneList.chr <- geneList[which(geneList[,1] == chr),]
#     
#     geneList.chr  <- geneList.chr[order(geneList.chr[,2],geneList.chr[,3]),]
#     
#     intersect <- geneList.chr[which(geneList.chr[,2] <= pos & geneList.chr[,3] >= pos),]
#     
#     if (nrow(intersect) > 0){
#       outTable$Gene.id <-  intersect[1,4]
#       outTable$chr <-  intersect[1,1]
#       outTable$start <-  intersect[1,2]
#       outTable$end <-  intersect[1,3]
#       outTable$dist <- 1
#     }else{
#       endBefore <- tail(geneList.chr[which(geneList.chr[,3] <= pos),],n=1)
#       startAfter <- head(geneList.chr[which(geneList.chr[,2] >= pos),],n=1)
#       #the pos is near a gene
#       
#       if (nrow(endBefore) == 0){
#         outTable$Gene.id <- startAfter[,4]
#         outTable$chr <- startAfter[,1]
#         outTable$start <- startAfter[,2]
#         outTable$end <- startAfter[,3]
#         outTable$dist <- startAfter[,2] - pos
#       }else if (nrow(startAfter) == 0){
#         outTable$Gene.id <- endBefore[,4]
#         outTable$chr <- endBefore[,1]
#         outTable$start <- endBefore[,2]
#         outTable$end <- endBefore[,3]
#         outTable$dist <- pos - endBefore[,3]
#       }else if (pos - endBefore[,3] <= startAfter[,2] - pos){
#         outTable$Gene.id <- endBefore[,4]
#         outTable$chr <- endBefore[,1]
#         outTable$start <- endBefore[,2]
#         outTable$end <- endBefore[,3]
#         outTable$dist <- pos - endBefore[,3]
#       }else{
#         outTable$Gene.id <- startAfter[,4]
#         outTable$chr <- startAfter[,1]
#         outTable$start <- startAfter[,2]
#         outTable$end <- startAfter[,3]
#         outTable$dist <- startAfter[,2] - pos
#       } 
#     }
#     
#     return(outTable)
#   }
#   bp1.gene <- findNearestGene(chr=bp$chr1,pos=bp$pos1,geneList=geneList)
#   bp2.gene <- findNearestGene(chr=bp$chr2,pos=bp$pos2,geneList=geneList)  
#   outTableSeg1$Gene.id <- bp1.gene$Gene.id
#   outTableSeg1$gene.dist <- bp1.gene$dist  
#   outTableSeg2$Gene.id <- bp2.gene$Gene.id
#   outTableSeg2$gene.dist <- bp2.gene$dist
#   
#   
#   ## load segments intersecting bps
#   clonalityTable.this <- clonalityTable[which(clonalityTable$sample == bp$sample),]  
#   if ( nrow(clonalityTable.this) == 0 ){
#     return(rbind(outTableSeg1,outTableSeg2))
#   }  
#   segments.this <- cbind(clonalityTable.this$chr,clonalityTable.this$start,clonalityTable.this$end)
#   #non overlapping segs
#   segments.this[,3] <- segments.this[,3] - 1
#   bp1seg <- getSegmentsPos(chr=bp$chr1,initialPos=bp$pos1,finalPos=bp$pos1,segments.this)
#   bp2seg <- getSegmentsPos(chr=bp$chr2,initialPos=bp$pos2,finalPos=bp$pos2,segments.this)
#   if( length(bp1seg) != 1 || length(bp2seg) != 1   ){ 
#     err<- paste0("Error in BP ",toString(bp)," - line number ",bpLine,"\n")
#     cat(err)
#     #  stop(err)
#   }    
#   seg1 <- clonalityTable.this[bp1seg,]
#   seg2 <- clonalityTable.this[bp2seg,]
#   
#   ## add data from segments
#   
#   ## seg1
#   outTableSeg1$adm.global <- seg1$adm.global
#   outTableSeg1$seg.start <- seg1$start
#   outTableSeg1$seg.end <- seg1$end
#   outTableSeg1$log2 <- seg1$log2.corr
#   outTableSeg1$perc.overlap <- 1
#   outTableSeg1$integer.cn <- seg1$integerCN
#   ## seg2
#   outTableSeg2$adm.global <- seg2$adm.global
#   outTableSeg2$seg.start <- seg2$start
#   outTableSeg2$seg.end <- seg2$end
#   outTableSeg2$log2 <- seg2$log2.corr
#   outTableSeg2$perc.overlap <- 1
#   outTableSeg2$integer.cn <- seg2$integerCN
#   
#   getBeta <- function(adm,cn){
#     return( (2 * adm) / ( 2 * adm + cn - cn * adm ) )
#   }
#   
#   ## if integer CN is defined compute AP statistics
#   if (!is.na(outTableSeg1$integer.cn) && !is.na(outTableSeg2$integer.cn)){    
#     ## compute AP and AP corrected
#     bp1Den <- bp$breakPointer.cov + bp$BP1.cov
#     bp2Den <- bp$breakPointer.cov + bp$BP2.cov
#     
#     beta1 <- getBeta(seg1$adm.global, outTableSeg1$integer.cn)
#     beta1.min <- getBeta(seg1$adm.global.min, outTableSeg1$integer.cn)
#     beta1.max <- getBeta(seg1$adm.global.max, outTableSeg1$integer.cn)      
#     bp1DenCorr <- bp$breakPointer.cov + max(bp$BP1.cov - round(bp1Den * ( beta1  )),0)
#     bp1DenCorrMin <- bp$breakPointer.cov + max(bp$BP1.cov - round(bp1Den * ( beta1.max )),0)
#     bp1DenCorrMax <- bp$breakPointer.cov + max(bp$BP1.cov - round(bp1Den * ( beta1.min )),0)
#     bp1AP <- bp$breakPointer.cov / bp1DenCorr
#     bp1MinAP <- bp$breakPointer.cov / bp1DenCorrMax
#     bp1MaxAP <- bp$breakPointer.cov / bp1DenCorrMin
#     
#     beta2 <- getBeta(seg2$adm.global, outTableSeg2$integer.cn)
#     beta2.min <- getBeta(seg2$adm.global.min, outTableSeg2$integer.cn)
#     beta2.max <- getBeta(seg2$adm.global.max, outTableSeg2$integer.cn)                  
#     bp2DenCorr <- bp$breakPointer.cov + max(bp$BP2.cov - round(bp2Den * ( beta2 )),0)
#     bp2DenCorrMin <- bp$breakPointer.cov + max(bp$BP2.cov - round(bp2Den * ( beta2.max )),0)
#     bp2DenCorrMax <- bp$breakPointer.cov + max(bp$BP2.cov - round(bp2Den * ( beta2.min )),0)
#     bp2AP <- bp$breakPointer.cov / bp2DenCorr
#     bp2MinAP <- bp$breakPointer.cov / bp2DenCorrMax
#     bp2MaxAP <- bp$breakPointer.cov / bp2DenCorrMin
#     
#     
#     outTableSeg1$clonality <- bp1AP
#     outTableSeg1$clonality.min <- bp1MinAP
#     outTableSeg1$clonality.max <- bp1MaxAP
#     
#     outTableSeg2$clonality <- bp2AP
#     outTableSeg2$clonality.min <- bp2MinAP
#     outTableSeg2$clonality.max <- bp2MaxAP
#     
#     if ( is.nan(bp1AP) || is.nan(bp2AP) ) {
#       return(rbind(outTableSeg1,outTableSeg2))
#     }
#     
#     #compute thresholds 
#     if ( bp1AP <= 0.5){
#       AP1threshold <- (qbinom((p.val/2),bp1DenCorr,0.5))/bp1DenCorr        
#     }else{
#       AP1threshold <- (qbinom(1-(p.val/2),bp1DenCorr,0.5))/bp1DenCorr        
#     }
#     if ( bp2AP <= 0.5){
#       AP2threshold <- (qbinom((p.val/2),bp2DenCorr,0.5))/bp2DenCorr        
#     }else{
#       AP2threshold <- (qbinom(1-(p.val/2),bp2DenCorr,0.5))/bp2DenCorr        
#     }            
#     outTableSeg1$APthr <- AP1threshold
#     outTableSeg2$APthr <- AP2threshold
#     
#     #compute status    
#     if ( outTableSeg1$integer.cn == 2 ){
#       if ( outTableSeg1$APthr <= 0.5 ){
#         if (outTableSeg1$clonality.min >= AP1threshold && outTableSeg1$clonality.max >= AP1threshold){ outTableSeg1$clonality.status <- "clonal" }
#         if (outTableSeg1$clonality.min < AP1threshold && outTableSeg1$clonality.max < AP1threshold){ outTableSeg1$clonality.status <- "subclonal" }
#         if (outTableSeg1$clonality.min < AP1threshold && outTableSeg1$clonality.max >= AP1threshold && outTableSeg1$clonality >= AP1threshold){ outTableSeg1$clonality.status <- "uncertain.clonal" }
#         if (outTableSeg1$clonality.min < AP1threshold && outTableSeg1$clonality.max >= AP1threshold && outTableSeg1$clonality < AP1threshold){ outTableSeg1$clonality.status <- "uncertain.subclonal" }          
#       }else{
#         if (outTableSeg1$clonality.min <= AP1threshold && outTableSeg1$clonality.max <= AP1threshold){ outTableSeg1$clonality.status <- "clonal" }
#         if (outTableSeg1$clonality.min > AP1threshold && outTableSeg1$clonality.max > AP1threshold){ outTableSeg1$clonality.status <- "subclonal" }
#         if (outTableSeg1$clonality.min <= AP1threshold && outTableSeg1$clonality.max > AP1threshold && outTableSeg1$clonality <= AP1threshold){ outTableSeg1$clonality.status <- "uncertain.clonal" }
#         if (outTableSeg1$clonality.min <= AP1threshold && outTableSeg1$clonality.max > AP1threshold && outTableSeg1$clonality > AP1threshold){ outTableSeg1$clonality.status <- "uncertain.subclonal" }        
#       }
#     }
#     
#     if ( outTableSeg2$integer.cn == 2 ){
#       if ( outTableSeg2$APthr <= 0.5 ){
#         if (outTableSeg2$clonality.min >= AP2threshold && outTableSeg2$clonality.max >= AP2threshold){ outTableSeg2$clonality.status <- "clonal" }
#         if (outTableSeg2$clonality.min < AP2threshold && outTableSeg2$clonality.max < AP2threshold){ outTableSeg2$clonality.status <- "subclonal" }
#         if (outTableSeg2$clonality.min < AP2threshold && outTableSeg2$clonality.max >= AP2threshold && outTableSeg2$clonality >= AP2threshold){ outTableSeg2$clonality.status <- "uncertain.clonal" }
#         if (outTableSeg2$clonality.min < AP2threshold && outTableSeg2$clonality.max >= AP2threshold && outTableSeg2$clonality < AP2threshold){ outTableSeg2$clonality.status <- "uncertain.subclonal" }          
#       }else{
#         if (outTableSeg2$clonality.min <= AP2threshold && outTableSeg2$clonality.max <= AP2threshold){ outTableSeg2$clonality.status <- "clonal" }
#         if (outTableSeg2$clonality.min > AP2threshold && outTableSeg2$clonality.max > AP2threshold){ outTableSeg2$clonality.status <- "subclonal" }
#         if (outTableSeg2$clonality.min <= AP2threshold && outTableSeg2$clonality.max > AP2threshold && outTableSeg2$clonality <= AP2threshold){ outTableSeg2$clonality.status <- "uncertain.clonal" }
#         if (outTableSeg2$clonality.min <= AP2threshold && outTableSeg2$clonality.max > AP2threshold && outTableSeg2$clonality > AP2threshold){ outTableSeg2$clonality.status <- "uncertain.subclonal" }        
#       }
#     }
#   }
#   
#   return(rbind(outTableSeg1,outTableSeg2))
#   
# }

