
cat(paste("[",Sys.time() ,"] Apply ploidy correction\n",sep=""))
betaTableOriginal <- betaTable
betaTable$log2 <- ploidyShift(betaTable=betaTable,correctionTable=ploidyTable)

cat(paste("[",Sys.time() ,"] Extract relevant deletions\n",sep=""))
betaTable.bySample <- by(betaTable,betaTable$sample,identity)


#select good deletions and (eventually) good gains
globalAdm.goodDels <-  mclapply(X=betaTable.bySample,
                                getRelevantBetasDels_AdmGlobal,
                                deletionsLog2Levels=deletionsLog2Levels,
                                min_nsnps=min_nsnps,
                                min_cov=min_cov,
                                errorTable=errorTable,
                                mc.preschedule=F,                                  
                                mc.cores=NsamplesToProcessInParallel)

# if (length(deletionsLog2Levels) == 4){
#   globalAdm.goodGains <-  mclapply(X=betaTable.bySample,
#                                   getRelevantBetasGains_AdmGlobal,
#                                   deletionsLog2Levels=deletionsLog2Levels,
#                                   min_nsnps=min_nsnps,
#                                   min_cov=min_cov,
#                                   errorTable=errorTable,
#                                   mc.preschedule=F,                                  
#                                   mc.cores=n_samples_in_parallel)
# }

cat(paste("[",Sys.time() ,"] Compute global admixture\n",sep=""))

globalAdm <- data.frame(row.names=unique(betaTable$sample),stringsAsFactors=F)
globalAdm$sample <- rownames(globalAdm)
globalAdm$adm <- NA
globalAdm$adm.min <- NA
globalAdm$adm.max <- NA
globalAdm$n.segs <- NA
globalAdm$n.SNPs <- NA

#globalAdm.goodDels[[4]]
#sampleName <- names(globalAdm.goodDels)[4]
for(sampleName in names(globalAdm.goodDels)){
  thisData <- globalAdm.goodDels[[sampleName]]
  if(is.null(thisData)){next}
  
  globalAdm[sampleName,"n.segs"] <- nrow(thisData)
  globalAdm[sampleName,"n.SNPs"] <- sum(thisData$nsnp,na.rm=T)
  
  if ( nrow(thisData) > 4 ){
    beta.median <- median(thisData$beta)
    beta.min <- quantile(ecdf(thisData$beta),probs=((1 -alphaPar)/2))
    beta.max <- quantile(ecdf(thisData$beta),probs=alphaPar+((1 -alphaPar)/2))    
  }else if ( nrow(thisData) < 2 ){
    beta.median <- thisData$beta
    beta.min <- max(0,beta.median-thisData$errorBeta)
    beta.max <- min(1,beta.median+thisData$errorBeta)
  }else{
    beta.median <- median(thisData$beta)
    maxError <- max(thisData$errorBeta)
    minVal <- quantile(ecdf(thisData$beta),probs=((1 -alphaPar)/2))
    maxVal <- quantile(ecdf(thisData$beta),probs=alphaPar+((1 -alphaPar)/2))    
    if ((maxVal - minVal) > (2*maxError)){
      beta.min <- minVal
      beta.max <- maxVal
    }else{
      beta.min <- max(0,beta.median-maxError)
      beta.max <- min(1,beta.median+maxError)
    }
    
    
  }
  
  globalAdm[sampleName,"adm"] <- beta.median / (2 - beta.median)
  globalAdm[sampleName,"adm.min"] <- beta.min / (2 - beta.min)
  globalAdm[sampleName,"adm.max"] <- beta.max / (2 - beta.max)
  
}

cat(paste("[",Sys.time() ,"] Save globald DNA admixture table in ",admTable_file,"\n",sep=""))
write.table(x=globalAdm,admTable_file,quote=F,sep="\t",col.names=T,row.names=F)


cat(paste("[",Sys.time() ,"] Save globald DNA admixture report in ",reportFile,"\n",sep=""))

pdf(file=reportFile,width=11,height=8)
par(mfrow=c(1,3),oma=c(0,0,3,0),mar=c(5,4,1,2))
sampleName <- names(globalAdm.goodDels)[1]
for(sampleName in names(globalAdm.goodDels)){
  
  this.segs <- betaTable[ which( betaTable$sample == sampleName & !is.na(betaTable$beta) & betaTable$cov >= min_cov &  betaTable$nsnp >= min_nsnps),]
  
  allData <- betaTable[
    which(
      betaTable$sample == sampleName &
        !is.na(betaTable$beta) & 
        betaTable$cov >= min_cov &  
        betaTable$nsnp >= min_nsnps & betaTable$log2 >= deletionsLog2Levels[1] & betaTable$log2 <= deletionsLog2Levels[2] 
    ),]
  
  thisData <- globalAdm.goodDels[[sampleName]]
  
  if(is.null(thisData)){next}
  
  if ( nrow(thisData) > 4 ){
    beta.median <- median(thisData$beta)
    beta.min <- quantile(ecdf(thisData$beta),probs=((1 -alphaPar)/2))
    beta.max <- quantile(ecdf(thisData$beta),probs=alphaPar+((1 -alphaPar)/2))    
  }else if ( nrow(thisData) < 2 ){
    beta.median <- thisData$beta
    beta.min <- max(0,beta.median-thisData$errorBeta)
    beta.max <- min(1,beta.median+thisData$errorBeta)
  }else{
    beta.median <- median(thisData$beta)
    maxError <- max(thisData$errorBeta)
    minVal <- quantile(ecdf(thisData$beta),probs=((1 -alphaPar)/2))
    maxVal <- quantile(ecdf(thisData$beta),probs=alphaPar+((1 -alphaPar)/2))    
    if ((maxVal - minVal) > (2*maxError)){
      beta.min <- minVal
      beta.max <- maxVal
    }else{
      beta.min <- max(0,beta.median-maxError)
      beta.max <- min(1,beta.median+maxError)
    }
    
  }
  
  thisData$adm.local <- thisData$beta / (2- thisData$beta)
  allData$adm.local <- allData$beta / (2- allData$beta)
  
  adm.median <- beta.median / (2 - beta.median)
  adm.min  <- beta.min / (2 - beta.min)
  adm.max <- beta.max / (2 - beta.max)
  
  plot(this.segs$log2,this.segs$beta,xlim=c(-2,2),ylim=c(0,1),pch=20,ylab="Beta",xlab="log2",main="All Segments")
  abline(v=log2(seq(1,3)/2),col="blue4")
  abline(h=c(beta.min,beta.median,beta.max),col="blue4")  
  
  plot(allData$log2,allData$beta,xlim=c(-2,2),ylim=c(0,1),pch=20,ylab="Beta",xlab="log2",main="Good Segments")
  abline(v=log2(seq(1,3)/2),col="blue4")
  abline(h=c(beta.min,beta.median,beta.max),col="blue4")  
  
  plot(thisData$log2,thisData$beta,xlim=c(-1.5,1.5),ylim=c(0,1),pch=20,ylab="Beta",xlab="log2",main="Used Segments")
  abline(v=log2(seq(1,3)/2),col="blue4")
  abline(h=c(beta.min,beta.median,beta.max),col="blue4")  
  mtext(text=sampleName,side=3,outer=T,font=2,cex=1.5,line=1)
  
  
}
dev.off()
  
betaTable <- betaTableOriginal


#plot(betaTable.bySample[[2]]$log2, betaTable.bySample[[2]]$beta )

###
#
#








#computeCTMglobal.perSample(betaTable.bySample[[1]],deletionsLog2Levels,min_nsnps=min_nsnps_cn1,min_cov,genes.cov.computed=T,genes.log2.computed=T)


