#!/usr/bin/env Rscript

######################################
## Load configuration file
cmd_args <- commandArgs(trailingOnly = T)
if (length(cmd_args) != 1){
  cat(paste0("[",Sys.time() ,"] ERROR - Usage: CLONET.analyzGeneSCNA.R <config_file>\n"))
  quit()
}

######################################
## Plot intro message
ClonET_version<-"1.0.0"
cat("\n")
cat(paste0("[",Sys.time() ,"] CLONET v",ClonET_version,"\n",sep=""))
cat(paste0("[",Sys.time() ,"]   Tool to analyze CN status of genes\n",sep=""))
wd <- getwd()
cat(paste0("[",Sys.time() ,"] Working directory ",wd,"\n\n",sep=""))


######################################
## Load configuration file
configFile <- cmd_args[1]
cat(paste0("[",Sys.time() ,"] Load configuration file: ",configFile,"\n",sep=""))
source(configFile)

######################################
## Check R version
minMajorVersion <- 2
minMinorVersion <- 7 
currect_R_version <- R.version
if ( ( as.numeric(currect_R_version$major) < minMajorVersion)  || 
       ( as.numeric(currect_R_version$major) == minMajorVersion && as.numeric(currect_R_version$minor) < minMinorVersion) ){
  cat(paste0("[",Sys.time() ,"] ERROR - CLONET require at least version ",minMajorVersion,".",minMinorVersion," but you have version ",currect_R_version$major,".",currect_R_version$minor, "\n"))
  quit()
}


######################################
## Check installed packages
cat(paste0("[",Sys.time() ,"] Check installed packages\n",sep=""))
pkg.list = c("parallel","pso","dgof","sets")
pkg.list.match=match(pkg.list,installed.packages()[,"Package"],nomatch=FALSE)
pkg.list.match = !as.logical(pkg.list.match)
if(sum(pkg.list.match)>0) { install.packages(pkg.list[pkg.list.match],repos="http://cran.r-project.org") } 
library(parallel,warn.conflicts=F)
library(pso,warn.conflicts=F)
library(dgof,warn.conflicts=F)
library(sets,warn.conflicts=F)
options(scipen=99999)

######################################
## Load CLONET basic functions
cat(paste0("[",Sys.time() ,"] Load CLONET basic functions\n",sep=""))
source(paste0(path_to_CLONET_functions,"Functions/CLONET.BasicFunctions.R"))

######################################
## Check output directory
output_DIR <- dirname(OutTable_file)
cat(paste("[",Sys.time() ,"] Output directory ",output_DIR,"\n",sep=""))
if (!file.exists(output_DIR)){
  cat(paste("[",Sys.time() ,"] WARNING - Output directory does not exit and will be created\n",sep=""))
  dir.create(path=output_DIR)
}




#####################################
## Computing per per gene SCNA clonality value
cat("\n\n######################################################################\n")
cat(paste("[",Sys.time() ,"] Compute per gene SCNA clonality table\n",sep=""))
cat(paste("[",Sys.time() ,"] Genes to analyze ",geneList_file,"\n",sep=""))
cat(paste("[",Sys.time() ,"] CLONET clonality table ",ClonalityTable_file,"\n",sep=""))
cat(paste("[",Sys.time() ,"] Output file name ",OutTable_file,"\n",sep=""))


######################################
##Load data
cat(paste("[",Sys.time() ,"] Loading data\n",sep=""))
clonalityTable <- read.csv(ClonalityTable_file,header=T,sep="\t",as.is=T)
adm.global <- read.csv(adm.global_file,header=T,sep="\t",as.is=T)
geneList <- read.csv(geneList_file,header=F,sep="\t",as.is=T)

cat(paste("[",Sys.time() ,"] Compute summary data\n",sep=""))
perGeneAnalysis.list <- mclapply(c(1:nrow(geneList)),FUN=getGeneStatus,geneList,clonalityTable,mc.preschedule=T,mc.cores=ncores)

perGeneAnalysis.list[sapply(perGeneAnalysis.list, is.null)] <- NULL



if (length(perGeneAnalysis.list) > 0 ){
  perGeneAnalysis <- fromListToDF(perGeneAnalysis.list)
  cat(paste("[",Sys.time() ,"] Write results\n",sep=""))
  perGeneAnalysis <- as.data.frame(lapply(perGeneAnalysis,unlist))
  write.table(perGeneAnalysis,file=OutTable_file,quote=F,sep="\t",row.names=F,col.names=T)
  
  Today <- format(Sys.time(), "%a, %b %d")
  Hour <- format(Sys.time(), "%H:%M")
  cat(paste0("Computation end on ",Today," at ",Hour,"\n"))
}else{
  
  cat(paste("[",Sys.time() ,"] CLONET did not find any valid gene\n",sep=""))
  Today <- format(Sys.time(), "%a, %b %d")
  Hour <- format(Sys.time(), "%H:%M")
  cat(paste0("Computation end on ",Today," at ",Hour,"\n"))
}
